# encoding: utf8
import logging
from ..services.plotters.png import ServicePlotterPNG
#from ..services.plotters.html import ServicePlotterHTML

logger = logging.getLogger(__name__)


class ServicePlotterFactory:
    """
    Factory to builds ServicePlotter objects
    """

    FORMAT_PNG = 'png'
    FORMAT_HTML = 'html'

    @classmethod
    def factory(cls, plot_format):
        """
        Created a ServicePlotter object

        :param plot_format: The plotting format
        :return: A ServicePlotter initialized object
        :rtype: ServicePlotter
        """
        logger.debug('ServicePlotterFactory.factory(%s)' % plot_format)
        if plot_format == cls.FORMAT_PNG:
            return ServicePlotterPNG
        #elif plot_format == cls.FORMAT_HTML:
        #    return ServicePlotterHTML
        else:
            raise Exception("Format '%s' is not supported by any plotter" % plot_format)



# encoding: utf8
from datetimerange import DateTimeRange
from resif_delivery_stats_plotter.services.clients.availabilty import ServiceClientAvailability as service_availability
from resif_delivery_stats_plotter.services.clients.statistics import ServiceClientStatistics as service_stats
from .enums import EnumChannelBand, EnumChannelInstrument


class Channel:
    """
    Defines a channel
    """

    def __init__(self, code=None, location=None):
        self._network_code = None
        self._station_code = None
        self.location = location
        self.code = code
        if code:
            split_code = list(code)
            self._band = split_code[0]
            self._instrument = split_code[1]
            self._orientation = split_code[2]

        self.timeslots = []

    def __repr__(self):
        return "<Channel %s (%i timeslots)>" % (
            self.location_code,
            len(self.timeslots)
        )

    def __lt__(self, other):
        """
        Check if the current channel is 'before' another
        Inspired by the work of Marc Grunberg (https://gitlab.com/eost/qc)

        :param other: An other channel to compare
        """
        if not isinstance(other, Channel):
            raise Exception("Unable to compare a 'Channel' object with '%s'" % type(other))

        def cmp(a, b):
            return (a > b) - (a < b)

        weight_sr = {"M": 9, "R": 8, "U": 7,
                     "V": 6, "L": 5, "B": 4,
                     "S": 3, "E": 2, "H": 1,
                     "C": 0, "D": -1}
        weight_gain = {"N": 2, "L": 1, "H": 0,
                       "V": 3, "D": 4, "T": 5,
                       "X": 6, "M": 7, "A": 8,
                       "W": 9, "E": 10, "K": 11}
        weight_cmp = {"E": 2, "N": 1, "Z": 0,
                      "3": 2, "2": 1, "1": 0.1,
                      "X": 1, "Y": 2,
                      "A": 3, "B": 4, "C": 5, "D": 6,
                      "O": 4, "V": 5, "S": 6}

        # check channel
        for i, w in zip((0, 1, 2), (weight_sr, weight_gain, weight_cmp)):
            try:
                res = cmp(w[self.code[i]], w[other.code[i]])
            except:
                # other code not yet handled, use lexico order only
                res = cmp(self.code[i], other.code[i])
            if res:
                break

        return res

    # CODES
    ####################################################################################################################

    @property
    def has_location(self):
        """
        Checks if the channel has a location

        :rtype: bool
        """
        return self.location is not None and self.location != ''

    @property
    def location_code(self):
        """
        Builds a full code including location if available

        :rtype: str
        """
        if self.has_location:
            return "%s.%s" % (self.location, self.code)
        else:
            return self.code

    @property
    def group_code(self):
        """
        Builds a custom code (without orientation)

        :rtype: str
        """
        return self.custom_code(
            with_location=True,
            with_band=True,
            with_instrument=True,
            with_orientation=False,
        )

    def custom_code(self, with_location=True, with_band=True, with_instrument=True, with_orientation=False):
        """
        Builds a custom code for the current channel

        :param with_location: Includes the location into the generated code
        :param with_band: Includes the band into the generated code
        :param with_instrument: Includes the instrument into the generated code
        :param with_orientation: Includes the orientation into the generated code
        :return: The generated custom code
        :rtype: str
        """
        code = ''
        if with_band:
            code += self._band
        if with_instrument:
            code += self._instrument
        if with_orientation:
            code += self._orientation

        if with_location and self.location:
            if code:
                code = '%s.%s' % (self.location, code)
            else:
                code += self.location

        return code

    @property
    def _instrument_orientation(self):
        """
        Gets the instrument and the orientation of the current channel

        :rtype: str
        """
        return "%s%s" % (self._instrument, self._orientation)

    @property
    def instrument(self):
        """
        Gets the instrument of the current channel

        :rtype: EnumChannelInstrument
        """
        return EnumChannelInstrument(self._instrument)

    @property
    def band(self):
        """
        Gets the band of the current channel

        :rtype: EnumChannelBand
        """
        return EnumChannelBand(self._band)

    # TIMESLOTS
    ####################################################################################################################

    def add_timeslot(self, timeslot):
        """
        Adds a timeslot to the current channel

        :param timeslot: The simeslot to add
        """
        self.timeslots.append(timeslot)

    def is_open(self, year):
        """
        Checks if the channel has been opened during a year

        :param year: The year to check
        :rtype: bool
        """
        for timeslot in self.timeslots:
            if timeslot.is_open(year):
                return True

        return False

    def get_timeslots(self, year):
        """
        Gets the timeslots of a year

        :param year: The year to filter
        :return: The list of timeslots opened during a specific year
        :rtype: list
        """
        timeslots = []

        for timeslot in self.timeslots:
            if timeslot.is_open(year):
                timeslots.append(timeslot)

        return timeslots

    def get_best_timeslot(self, year):
        """
        Gets the best timeslot of a year

        :param year: The year to filter
        :return: The timeslots with the longest opened period during a specific year
        :rtype: ChannelTimeslot
        """
        best_timeslot = None

        for timeslot in self.timeslots:
            if timeslot.is_open(year) and (best_timeslot is None or timeslot.date_range_intersection(year).timedelta > best_timeslot.date_range_intersection(year).timedelta):
                best_timeslot = timeslot

        return best_timeslot

    # STATISTICS
    ####################################################################################################################

    def requests(self, year=None, month=None, country=None, media=None):
        """
        Get the requests number for the current channel

        :param year: The year to filter the requests
        :param month: The month to filter the requests
        :param country: The country to filter the requests
        :param media: The media to filter the requests
        :return: The requests number
        :rtype: int
        """
        return service_stats.requests(network=self._network_code, station=self._station_code, location=self.location, channel=self.code, year=year, month=month, country=country, media=media)

    def requests_by_country(self, year=None, month=None, country=None, media=None):
        """
        Get the requests number for each country, for the current channel

        :param year: The year to filter the requests
        :param month: The month to filter the requests
        :param country: The country to filter the requests
        :param media: The media to filter the requests
        :return: The requests number
        :rtype: int
        """
        return service_stats.requests_by_country(network=self._network_code, station=self._station_code, location=self.location, channel=self.code, year=year, month=month, country=country, media=media)

    def clients(self, year=None, month=None, country=None, media=None):
        """
        Get the clients number for each country, for the current channel

        :param year: The year to filter the clients
        :param month: The month to filter the clients
        :param country: The country to filter the clients
        :param media: The media to filter the clients
        :return: The clients number
        :rtype: int
        """
        return service_stats.clients(network=self._network_code, station=self._station_code, location=self.location, channel=self.code, year=year, month=month, country=country, media=media)

    def clients_by_country(self, year=None, month=None, country=None, media=None):
        """
        Gets the amount of data send for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param country: The country to filter the stats
        :param media: The media to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send
        :rtype: int
        """
        return service_stats.clients_by_country(network=self._network_code, station=self._station_code, location=self.location, channel=self.code, year=year, month=month, country=country, media=media)

    def data_send(self, year=None, month=None, country=None, media=None, unit=None):
        """
        Gets the amount of data send by dataselect for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param country: The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by dataselect
        :rtype: int
        """
        return service_stats.data_send(network=self._network_code, station=self._station_code, location=self.location, channel=self.code, year=year, month=month, country=country, media=media, unit=unit)

    def data_send_dataselect(self, year=None, month=None, country=None, unit=None):
        """
        Gets the amount of data send by dataselect for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param country: The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by dataselect
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='dataselect', unit=unit)

    def data_send_seedlink(self, year, month=None, country=None, unit=None):
        """
        Gets the amount of data send by seedlink for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param country: The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by seedlink
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='seedlink', unit=unit)

    def data_send_station(self, year, month=None, country=None, unit=None):
        """
        Gets the amount of data send by station WS for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param country: The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by station WS
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='station', unit=unit)

    def availability(self, year, month=None, quality=None):
        """
        Gets the availability periods for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param quality: The quality to filter the stats
        :return: The amount of data send by station WS
        :rtype: list
        """
        return service_availability.channel(self.code, self.location, self._station_code, self._network_code, year, month, quality)

    def check_availability(self, year, month=None, quality=None):
        """
        Checks the availability periods for the current channel

        :param year: The year to filter the stats
        :param month: The month to filter the stats
        :param quality: The quality to filter the stats
        :return: Boolean flag telling if 'availability' stats are available for this channel
        :rtype: bool
        """
        return service_availability.check_channel(self.code, self.location, self._station_code, self._network_code, year, month, quality)


class ChannelTimeslot:
    """
    Defines a timeslot for a channel
    """

    def __init__(self, start_date, end_date):
        if isinstance(start_date, tuple):
            self._start_date = start_date[0]
        else:
            self._start_date = start_date

        if isinstance(end_date, tuple):
            self._end_date = end_date[0]
        else:
            self._end_date = end_date

    def __repr__(self):
        return "<ChannelTimeslot (%s - %s)>" % (
            self.date_start, self.date_end
        )

    @property
    def date_start(self):
        """
        Gets the open date in ISO format for the current timeslot

        :rtype str
        """
        if self._start_date:
            return self._start_date.isoformat()

    @property
    def date_end(self):
        """
        Gets the close date in ISO format for the current timeslot

        :rtype str
        """
        if self._end_date:
            return self._end_date.isoformat()

    @property
    def date_range_full(self):
        """
        Gets the full date range for the current timeslot

        :rtype DateTimeRange
        """
        return DateTimeRange(self.date_start, self.date_end)

    def date_range_intersection(self, year):
        """
        Gets the intersection date range for the current timeslot with a specific year

        :rtype DateTimeRange
        """
        return self.date_range_full.intersection(DateTimeRange("%i-01-01" % year, "%i-12-31" % year))

    def is_open(self, year):
        """
        Checks if the channel timeslot was during a specific year

        :param year: The year to check
        :rtype: bool
        """
        return self.date_range_full.is_intersection(DateTimeRange("%i-01-01" % year, "%i-12-31" % year))



# encoding: utf8
import logging
from datetime import date
from obspy.clients.fdsn import Client
from obspy.clients.fdsn.header import FDSNNoDataException, FDSNException
from .enums import EnumObspyLevel
from ..errors import NoDataError, ApiError
from ..models.channel import Channel, ChannelTimeslot

logger = logging.getLogger(__name__)


class ChannelFactory:
    """
    Factory to builds Channel objects from an ObsPy instance
    """

    @classmethod
    def from_obspy(cls, obspy_channel, with_timeslot=True, year=None):
        """
        Create a Channel object from an ObsPy instance

        :param obspy_channel: The ObsPy instance
        :param with_timeslot: Creates the first ChannelTimeslot object
        :param year: The year to filter opened timeslots
        :return: A Channel initialized object
        :rtype: Channel
        """
        logger.debug('ChannelFactory.from_obspy(%s, %s, %s)' % (obspy_channel.code, with_timeslot, year))
        channel = Channel(
            code=obspy_channel.code,
            location=obspy_channel.location_code,
        )

        if with_timeslot:
            timeslot = ChannelTimeslotFactory.from_obspy(obspy_channel)
            if year is None or timeslot.is_open(year):
                channel.add_timeslot(timeslot)

        return channel

    @classmethod
    def location_code_from_obspy(cls, obspy_channel):
        """
        Generates a location code from an ObsPy instance

        :param obspy_channel: The ObsPy instance
        :return: The location code of the channel
        :rtype: str
        """
        if hasattr(obspy_channel, 'location_code') and obspy_channel.location_code not in (None, '', 'None'):
            return "%s.%s" % (obspy_channel.location_code, obspy_channel.code)
        else:
            return obspy_channel.code

    @classmethod
    def list(cls, code_network, code_station, level=EnumObspyLevel.channel, year=None):
        """
        Gets a Channel object from its code

        :param code_network: The code of the network to get
        :param code_station: The code of the station to get
        :param level: The ObsPy query level (default='channel')
        :param year: The year to filter opened stations and channels
        :return: A Network initialized object
        :rtype: Network
        """
        logger.debug('ChannelFactory.list(%s, %s, %s, %s)' % (code_network, code_station, level, year))
        try:
            params = {
                'network': code_network,
                'station': code_station,
                'level': level,
            }

            if year:
                params['starttime'] = date(year, 1, 1)
                params['endtime'] = date(year, 12, 31)

            channels = []
            for obspy_network in Client('RESIF').get_stations(**params):
                for obspy_station in obspy_network.stations:
                    for obspy_channel in obspy_station.channels:
                        channel = cls.from_obspy(obspy_channel, year=year)
                        channels.append(channel)
            return channels

        except FDSNNoDataException:
            logger.warning("No channel found for station %s.%s" % (code_network, code_station))
            raise NoDataError()
        except FDSNException as e:
            raise ApiError(e)


class ChannelTimeslotFactory:
    """
    Factory to builds ChannelTimeslot objects from an ObsPy instance of Channel
    """

    @classmethod
    def from_obspy(cls, obspy_channel):
        """
        Create a ChannelTimeslot object from an ObsPy instance

        :param obspy_channel: The ObsPy instance
        :return: A ChannelTimeslot initialized object
        :rtype: ChannelTimeslot
        """
        logger.debug('ChannelTimeslotFactory.from_obspy(<%s, %s, %s>)' % (obspy_channel.code, obspy_channel.start_date, obspy_channel.end_date))
        timeslot = ChannelTimeslot(
            start_date=obspy_channel.start_date,
            end_date=obspy_channel.end_date,
        )
        return timeslot



# encoding: utf8
import os
import logging
from .. import ServiceAbstract

logger = logging.getLogger(__name__)


class ServiceReportBuilderAbstract(ServiceAbstract):
    """
    Base class of report builders
    """

    @classmethod
    def build(cls, report):
        return NotImplemented

    @staticmethod
    def output_dir(report, create=False):
        """
        Builds, and creates, an output path for a report
        :param report: The report object
        :param create: Enables the creation of the folders tree if needed
        :return: The generated output path
        :rtype: str
        """
        logger.debug('ServiceReportBuilderAbstract.output_dir(%s, %s)' % (report, create))
        d = os.path.abspath(os.path.join('output', report.network.code, '%s' % report.year))

        if create:
            logger.debug('If needed, creating directory %s' % d)
            os.makedirs(d, exist_ok=True)

        return d



# encoding: utf8
import logging
from ..errors import NoDataError
from ..models.station import Station
from .channel import ChannelFactory
from .enums import EnumObspyLevel

logger = logging.getLogger(__name__)


class StationFactory:
    """
    Factory to builds Station objects from an ObsPy instance of Channel
    """

    @classmethod
    def from_obspy(cls, obspy_station, level, channel_year=None, network_code=None):
        """
        Create a Channel object from an ObsPy instance

        :param obspy_station: The ObsPy instance
        :param level: The ObsPy query level
        :param channel_year: The year to filter opened channels
        :return: A Station initialized object
        :rtype: Station
        """
        logger.debug('StationFactory.from_obspy(%s, %s, %s, %s)' % (obspy_station.code, level, channel_year, network_code))
        station = Station(
            code=obspy_station.code,
            start_date=obspy_station.start_date,
            end_date=obspy_station.end_date,
            latitude=obspy_station.latitude,
            longitude=obspy_station.longitude,
            elevation=obspy_station.elevation,
            network_code=network_code
        )

        if channel_year is not None and network_code is not None:
            try:
                for channel in ChannelFactory.list(network_code, obspy_station.code, year=channel_year):
                    station.add_channel(channel)
            except NoDataError:
                pass
        elif level == EnumObspyLevel.channel:
            for obspy_channel in obspy_station.channels:
                station.add_channel(ChannelFactory.from_obspy(obspy_channel, year=channel_year))

        return station



# encoding: utf8
from pint import Quantity


class ServiceAbstract:
    """
    Base class of services
    """

    @classmethod
    def unit_convert(cls, value, from_unit, to_unit):
        """
        Convert a value from an unit to another unit
        :param value: The value to convert
        :param from_unit: The source unit
        :param to_unit: The destination unit
        :return: The converted value
        :rtype: int|float
        """
        quantity = Quantity(value, from_unit)
        return quantity.to(to_unit).magnitude



# encoding: utf8
import logging
from datetime import date
from resif_delivery_stats_plotter.services.clients.statistics import ServiceClientStatistics as service_stats
from .mixins import LocationCodeMixin

logger = logging.getLogger(__name__)


class Station(LocationCodeMixin):
    """
    Defines a station
    """

    def __init__(self, code=None, start_date=None, end_date=None, latitude=None, longitude=None, elevation=None, network_code=None):
        self._network_code = network_code
        self.code = code
        self.latitude = latitude
        self.longitude = longitude
        self.elevation = elevation
        self._locations = {}
        self._channels = {}

        if isinstance(start_date, tuple):
            self._start_date = start_date[0]
        else:
            self._start_date = start_date

        if isinstance(end_date, tuple):
            self._end_date = end_date[0]
        else:
            self._end_date = end_date

    def __repr__(self):
        return "<Station %s (%s | %s) (%s|%s|%s) (%i locations)>" % (
            self.code,
            self.date_start, self.date_end,
            self.latitude, self.longitude, self.elevation,
            len(self._locations)
        )

    def __lt__(self, other):
        if not isinstance(other, Station):
            raise Exception("Unable to compare a 'Station' object with '%s'" % type(other))
        return self.code < other.code

    @property
    def date_start(self):
        """
        Gets the open date in ISO format for the current station

        :rtype str
        """
        if self._start_date:
            return self._start_date.isoformat()

    @property
    def date_end(self):
        """
        Gets the close date in ISO format for the current station

        :rtype str
        """
        if self._end_date:
            return self._end_date.isoformat()

    def is_open(self, year):
        """
        Checks if the station has been opened during a year

        :param year: The year to check
        :rtype: bool
        """
        if self._start_date is not None and self._start_date > date(year, 12, 31):
            return False

        if self._end_date is not None and self._end_date < date(year, 1, 1):
            return False

        return True

    @property
    def is_still_active(self):
        """
        Check s if the station is still active
        :rtype: bool
        """
        return self.is_open(2500)

    # LOCATIONS
    ####################################################################################################################

    def get_location(self, code):
        """
        Gets, and initializes if needed, a sub-collection by location code from the current collection

        :param code: The location code
        :return: A collection of channels
        :type: StationLocation
        """
        if code not in self._locations:
            location = StationLocation(code)
            self._locations[location.code] = location
        else:
            location = self._locations.get(code)
        return location

    # CHANNELS
    ####################################################################################################################

    def filter_channels(self, location=None, band=None, instrument=None, orientation=None):
        """
        Filters channels of the current collection

        :param location: The code of the location to filter the channels
        :param band: The code of the band to filter the channels
        :param instrument: The code of the instrument to filter the channels
        :param orientation: The code of the instrument's orientation to filter the channels
        :return: A selection of channels for the current station
        :rtype: list
        """
        channels = []

        for channel in self.channels:
            if location:
                if isinstance(location, (list, tuple)):
                    if channel.location not in location:
                        continue
                else:
                    if channel.location != location:
                        continue

            if band:
                if isinstance(band, (list, tuple)):
                    if channel.band not in band:
                        continue
                else:
                    if channel.band != band:
                        continue

            if instrument:
                if isinstance(instrument, (list, tuple)):
                    if channel.instrument not in instrument:
                        continue
                else:
                    if channel.instrument != instrument:
                        continue

            if orientation:
                if isinstance(orientation, (list, tuple)):
                    if channel.orientation not in orientation:
                        continue
                else:
                    if channel.orientation != orientation:
                        continue

            channels.append(channel)

        return channels

    def active_channels(self, year):
        """
        Gets the active channels of the current station during a defined year

        :param year: The year during which the channels have to be opened
        :return: A selection of channels
        :rtype: dict
        """
        active_channels = {}

        def check_channels(channels):
            for channel in channels.values():
                if channel.is_open(year):
                    active_channels[channel.location_code] = channel

        if self._locations:
            for location in self._locations.values():
                check_channels(location.channels)
        else:
            check_channels(self._channels)

        return active_channels

    def grouped_active_channels(self, year, group_location=True, group_band=True, group_instrument=True, group_orientation=False):
        """
        Gets the active channels of the current station during a defined year, and grouped by '<location>.<band><instrument>'

        :param year: The year during which the channels have to be opened
        :param group_location: Use location code to group channels
        :param group_band: Use band code to group channels
        :param group_instrument: Use instrument code to group channels
        :param group_orientation: Use orientation code to group channels
        :return: A selection of channels
        :rtype: dict
        """
        groups = {}

        def check_channels(channels):
            for channel in channels.values():
                if channel.is_open(year):
                    group_code = channel.custom_code(group_location, group_band, group_instrument, group_orientation)
                    if group_code not in groups:
                        groups[group_code] = {}

                    groups[group_code][channel.location_code] = channel

        if self._locations:
            for location in self._locations.values():
                check_channels(location.channels)
        else:
            check_channels(self._channels)

        return groups

    def add_channel(self, channel):
        """
        Adds a channel to the current station

        :param channel: The channel to add
        """
        channel._network_code = self._network_code
        channel._station_code = self.code
        if channel.has_location:
            location = self.get_location(channel.location)
            location.add_channel(channel)
        else:
            self._channels[channel.code] = channel

    def get_channel(self, location_code):
        """
        Gets a channel by its location code from the current station

        :param location_code: The location code of the channel to get
        :return: A channel
        :rtype: Channel
        """
        loc = self.extract_location_from_location_code(location_code)
        if loc:
            location = self._locations.get(loc)
            if location:
                return location.get_channel(location_code)

        return self._channels.get(location_code)

    @property
    def channels(self):
        """
        Gets the channels of the current station

        :return: A list of channel
        :rtype: list
        """
        if self._locations:
            channels = []
            for location_channels in self._locations.values():
                channels.extend(location_channels.values())
            return sorted(channels)
        else:
            return sorted(self._channels.values())

    def representative_channels(self, year=None, month=None, quality=None):
        """
        Gets the representative channel of the current station

        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality to filter
        :return: A selection of representative channels
        :rtype: list
        """
        if self._locations:
            representative_channels = []
            for location in self._locations.values():
                location_channel = location.representative_channel(year, month, quality)
                if location_channel:
                    representative_channels.append(location_channel)

            if not representative_channels:
                logger.warning("No representative channel found for station %s" % self.code)
            return representative_channels

        elif self._channels:
            for channel in sorted(self._channels.values()):
                if channel.check_availability(year, month, quality):
                    return [channel, ]
            logger.warning("No representative channel found for station %s" % self.code)
        else:
            logger.warning("No representative channel found for station %s" % self.code)
            return []

    # STATISTICS
    ####################################################################################################################

    def requests(self, year=None, month=None, country=None, media=None):
        """
        Get the requests number for the current station

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :param media:  The media to filter the requests
        :return: The requests number
        :rtype: int
        """
        return service_stats.requests(network=self._network_code, station=self.code, year=year, month=month, country=country, media=media)

    def requests_by_country(self, year=None, month=None, country=None, media=None):
        """
        Get the requests number for each country, for the current station

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :param media:  The media to filter the requests
        :return: The requests number
        :rtype: int
        """
        return service_stats.requests_by_country(network=self._network_code, station=self.code, year=year, month=month, country=country, media=media)

    def clients(self, year=None, month=None, country=None, media=None):
        """
        Get the clients number for the current station

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :param media:  The media to filter the clients
        :return: The clients number
        :rtype: int
        """
        return service_stats.clients(network=self._network_code, station=self.code, year=year, month=month, country=country, media=media)

    def clients_by_country(self, year=None, month=None, country=None, media=None):
        """
        Get the clients number for each country, for the current station

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :param media:  The media to filter the clients
        :return: The clients number
        :rtype: int
        """
        return service_stats.clients_by_country(network=self._network_code, station=self.code, year=year, month=month, country=country, media=media)

    def data_send(self, year=None, month=None, country=None, media=None, unit=None):
        """
        Gets the amount of data send for the current station

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param media:  The media to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send
        :rtype: int
        """
        return service_stats.data_send(network=self._network_code, station=self.code, year=year, month=month, country=country, media=media, unit=unit)

    def data_send_dataselect(self, year=None, month=None, country=None, unit=None):
        """
        Gets the amount of data send by dataselect for the current station

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by dataselect
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='dataselect', unit=unit)

    def data_send_seedlink(self, year, month=None, country=None, unit=None):
        """
        Gets the amount of data send by seedlink for the current station

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by seedlink
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='seedlink', unit=unit)

    def data_send_station(self, year, month=None, country=None, unit=None):
        """
        Gets the amount of data send by station WS for the current station

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by station WS
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='station', unit=unit)


class StationLocation(LocationCodeMixin):
    """
    Defines a location to group channels of a station
    """

    def __init__(self, code):
        self.code = code
        self.channels = {}

    def __repr__(self):
        return "<StationLocation %s (%i channels)>" % (
            self.code,
            len(self.channels)
        )

    def add_channel(self, channel):
        """
        Adds a channel to the current collection

        :param channel: The channel to add
        """
        self.channels[channel.location_code] = channel

    def get_channel(self, location_code):
        """
        Gets a channel by its location code from the current collection

        :param location_code: The location code of the channel to get
        :return: A channel
        :rtype: Channel
        """
        return self.channels.get(location_code)

    def representative_channel(self, year=None, month=None, quality=None):
        """
        Gets the representative channel of the current collection

        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality to filter
        :return: The representative channel
        :rtype: Channel
        """
        if self.channels:
            for channel in sorted(self.channels.values()):
                if channel.check_availability(year, month, quality):
                    return channel

    def open_channels(self, year, as_list=False):
        """
        Gets the opened channels during a year

        :param year: The year during which the channels have to be opened
        :param as_list: Change the return type to list instead of dict
        :return: The channels opened during a year
        :rtype: dict|list
        """
        channels = {}
        for channel in sorted(self.channels.values()):
            if channel.is_open(year):
                channels[channel.location_code] = channel

        if as_list:
            return sorted(channels.values())

        return channels



# encoding: utf8
import logging
import os
import pathlib
import pint
import datetime
import arrow
import geopandas
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.patches as mpatches
import matplotlib.cm as mcolormap
from iso3166 import countries
from staticmap import StaticMap, IconMarker
from resif_delivery_stats_plotter.services.clients.statistics import ServiceClientStatistics as service_stats
from resif_delivery_stats_plotter.services.clients.availabilty import ServiceClientAvailability as service_availability
from . import ServicePlotterAbstract

logger = logging.getLogger(__name__)


class ServicePlotterPNG(ServicePlotterAbstract):
    """
    Static PNG plotter
    """

    DEFAULT_PX_WIDTH = 800
    DEFAULT_PX_HEIGHT = 600
    MINIMAL_PX_WIDTH = 380

    @classmethod
    def _footer_bar(cls, fig, text=None, fontsize=10, pad=5, xpos=20, ypos=7.5, rect_kw = {"facecolor":"grey", "edgecolor":None}, text_kw = {"color":"w"}):
        """
        Adds a footer bar to matplotlib plots
        :param fig: The matplotlib figure
        :param text: The text of the footer bar
        :param fontsize: The fontsize (default=10)
        :param pad: The padding of the footer bar (default=5)
        :param xpos: The X position of the footer bar (default=20)
        :param ypos: The Y position of the footer bar (default=7.5)
        :param rect_kw: The other parameters of the footer bar
        :param text_kw: The other parameters of the footer text
        """
        today = datetime.datetime.utcnow()
        text = text or "Generated by Résif on %s" % today.strftime("%Y-%m-%d at %Hh%M UTC")
        w, h = fig.get_size_inches()
        height = ((fontsize+2*pad)/fig.dpi)/h
        rect = plt.Rectangle((0, 0), 1, height, transform=fig.transFigure, clip_on=False, **rect_kw)
        fig.axes[0].add_patch(rect)
        fig.text(xpos/fig.dpi/w, ypos/fig.dpi/h, text, fontsize=fontsize, **text_kw)
        fig.subplots_adjust(bottom=fig.subplotpars.bottom+height)

    @classmethod
    def _colors(cls, colormap=None, number=None, default_colormap='tab20'):
        """
        Gets a list of colors from the selected matplotlib colormap
        :param colormap: The colormap
        :param number: The number of colors to return
        :param default_colormap: The default colormap
        :return: A list of colors
        :rtype: list
        """
        logger.debug("ServicePlotterPNG._colors(%s, %s, %s)" % (colormap, number, default_colormap))

        colormap = colormap or default_colormap
        colors = mcolormap.get_cmap(colormap, number).colors

        if number:
            logger.debug("Using %i colors from '%s' colormap: %s" % (number, colormap, colors))
        else:
            logger.debug("Using '%s' colormap" % colormap)

        return colors

    @classmethod
    def plot_network_availability(cls, network, year=None, width=None, height=None, vertical=False, instrument=None, dpi=100):
        """
        Plots the annual availability of each station of a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param vertical: Enables the vertical layout (default=false)
        :param instrument: The instrument code to filter
        :param dpi: The DPI (default=100)
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_network_availability(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, vertical, instrument))
        year = year or datetime.datetime.now().year
        logger.info("Plotting availability of network %s in %i" % (network.code, year))

        # Get plot data
        plot_data = {}
        for station in network.stations:
            for channel in station.representative_channels(year):
                station_data = {}
                qualities = channel.availability(year)
                if qualities:
                    for quality, timespans in qualities.items():
                        station_data[quality] = []
                        for timespan in timespans:
                            station_data[quality].append((arrow.get(timespan[0]).datetime, arrow.get(timespan[1]).datetime))

                if station_data:
                    plot_data["%s.%s.%s" % (network.code, station.code, channel.location_code)] = station_data

        nb_lines = len(plot_data)
        logger.debug("Got %i channels to plot" % nb_lines)

        # Calculate figure dimensions
        line_px = 15
        margin_px_top = 20
        margin_px_right = 10
        if vertical:
            margin_px_left = 100
            margin_px_bottom = 180
            width_px = width or max(cls.MINIMAL_PX_WIDTH, (line_px * nb_lines + margin_px_left + margin_px_right))
            height_px = height or cls.DEFAULT_PX_HEIGHT
        else:
            margin_px_left = 180
            margin_px_bottom = 100
            width_px = width or cls.DEFAULT_PX_WIDTH
            height_px = height or (line_px * nb_lines + margin_px_top + margin_px_bottom)

        width_in = width_px / dpi
        height_in = height_px / dpi

        margin_in_left = margin_px_left / dpi
        margin_in_right = margin_px_right / dpi
        margin_in_top = margin_px_top / dpi
        margin_in_bottom = margin_px_bottom / dpi

        x_plot = margin_in_left / width_in
        y_plot = margin_in_bottom / height_in
        w_plot = (width_in - margin_in_left - margin_in_right) / width_in
        h_plot = (height_in - margin_in_bottom - margin_in_top) / height_in

        # Initialize image
        logger.debug("Figure size (Pixels): w=%s, h=%s" % (width_px, height_px))
        fig = plt.figure(figsize=(width_in, height_in))
        ax = plt.axes([x_plot, y_plot, w_plot, h_plot])

        # Initialize colors
        colors = {}
        handles = []

        color = ax._get_lines.get_next_color()
        for quality in (service_availability.QUALITY_Q_CONTROLLED, service_availability.QUALITY_M_MODIFIED):
            colors[quality] = color
        handles.append(mpatches.Patch(color=color, label='Valid'))

        color = ax._get_lines.get_next_color()
        for quality in (service_availability.QUALITY_R_RAW, service_availability.QUALITY_D_INDETERMINATE):
            colors[quality] = color
        handles.append(mpatches.Patch(color=color, label='Raw'))

        # Plot data
        if vertical:
            for trace_id in sorted(plot_data.keys()):
                for quality in plot_data[trace_id]:
                    color = colors.get(quality)
                    trace = (trace_id, trace_id)
                    for segment in plot_data[trace_id][quality]:
                        ax.plot(trace, segment, color=color)
        else:
            for trace_id in sorted(plot_data.keys()):
                for quality in plot_data[trace_id]:
                    color = colors.get(quality)
                    trace = (trace_id, trace_id)
                    for segment in plot_data[trace_id][quality]:
                        ax.plot(segment, trace, color=color)

        # Configure axis
        ax.xaxis.set_tick_params(rotation=90)
        if vertical:
            ax.yaxis.set_major_formatter(mdates.DateFormatter("%Y-%m"))
            ax.yaxis.set_major_locator(mdates.MonthLocator())
            ax.yaxis.set_minor_locator(mdates.WeekdayLocator())
            ax.grid(axis='y')
            ax.invert_xaxis()
        else:
            ax.xaxis.set_major_formatter(mdates.DateFormatter("%Y-%m"))
            ax.xaxis.set_major_locator(mdates.MonthLocator())
            ax.xaxis.set_minor_locator(mdates.WeekdayLocator())
            ax.grid(axis='x')
            ax.invert_yaxis()

        # Add title
        #ax.set_title("%s %i" % (network.code, year), loc='left', fontsize=10)

        # Add legend
        ax.legend(handles=handles, bbox_to_anchor=(1, 1), loc='lower right', borderaxespad=0, ncol=2, frameon=False)

        # Add footer
        cls._footer_bar(fig)

        # Save file
        output_filename = 'plot_network_availability.png'
        output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
        fig.savefig(output_filepath, transparent=True)
        logger.info("Plot saved as %s" % output_filepath)

        return output_filename

    @classmethod
    def plot_data_send_yearly(cls, network, year=None, width=None, height=None, vertical=True, start_year=None, unit=None, colormap=None):
        """
        Plots the amount of data send yearly for a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param vertical: Enables the vertical layout (default=false)
        :param start_year: The starting year
        :param unit: The unit in which the values are converted
        :param colormap: The colormap to use
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_data_send_yearly(%s, %s, %s, %s, %s, %s, %s, %s)" % (network, year, width, height, vertical, start_year, unit, colormap))
        year = year or datetime.datetime.now().year
        logger.info("Plotting data send yearly by network %s until %i" % (network.code, year))

        ureg = pint.UnitRegistry()
        ureg.setup_matplotlib(True)

        # Calculate starting year
        if start_year:
            start_year = max(network.year_start, start_year)
        else:
            start_year = network.year_start
        logger.debug('Starting year: %s' % start_year)

        # Get plot data
        plot_data_years = []
        plot_data_dataselect = []
        plot_data_seedlink = []
        has_data = False
        for y in range(start_year, year+1):
            send_dataselect = network.data_send_dataselect(year=y)
            send_seedlink = network.data_send_seedlink(year=y)
            if has_data or send_dataselect or send_seedlink:
                has_data = True
                plot_data_years.append(arrow.get(y, 1, 1).datetime)
                plot_data_dataselect.append((send_dataselect or 0) * ureg.byte)
                plot_data_seedlink.append((send_seedlink or 0) * ureg.byte)

        # Build image
        if plot_data_years:
            logger.debug("Plotted years: %s" % len(plot_data_years))
            bar_width = 0.75 * ureg.year
            fig, ax = plt.subplots()

            # Calculate dimensions of image
            bar_px = 50
            margin_px_top = 20
            margin_px_right = 10
            margin_px_left_bottom = 100
            if vertical:
                width_px = width or max(cls.MINIMAL_PX_WIDTH, (bar_px * len(plot_data_years) + margin_px_left_bottom + margin_px_right))
                height_px = height or cls.DEFAULT_PX_HEIGHT
            else:
                width_px = width or cls.DEFAULT_PX_WIDTH
                height_px = height or (bar_px * len(plot_data_years) + margin_px_top + margin_px_left_bottom)

            logger.debug("Figure size (Pixels): w=%s, h=%s" % (width_px, height_px))
            fig.set_size_inches(width_px/fig.dpi, height_px/fig.dpi)

            # Initialize colors
            colors = cls._colors(colormap, 2)

            # Configure axis and plot data
            ax.xaxis.set_tick_params(rotation=90)
            if vertical:
                ax.yaxis.set_label_text('Data send (%s) for %s' % (unit or 'Bytes', network.code))
                if unit:
                    ax.yaxis.set_units(ureg.Unit(unit))
                ax.xaxis.set_major_formatter(mdates.DateFormatter("%Y"))
                ax.xaxis.set_major_locator(mdates.YearLocator())
                ax.bar(plot_data_years, plot_data_seedlink, bar_width, label='Seedlink', color=colors[0])
                ax.bar(plot_data_years, plot_data_dataselect, bar_width, bottom=plot_data_seedlink, label='Dataselect', color=colors[1])
            else:
                ax.xaxis.set_label_text('Data send (%s) for %s' % (unit or 'Bytes', network.code))
                if unit:
                    ax.xaxis.set_units(ureg.Unit(unit))
                ax.yaxis.set_major_formatter(mdates.DateFormatter("%Y"))
                ax.yaxis.set_major_locator(mdates.YearLocator())
                ax.barh(plot_data_years, plot_data_seedlink, bar_width, label='Seedlink', color=colors[0])
                ax.barh(plot_data_years, plot_data_dataselect, bar_width, left=plot_data_seedlink, label='Dataselect', color=colors[1])

            # Add title
            #ax.set_title(network.code, loc='left')

            # Add legend
            ax.legend(bbox_to_anchor=(1, 1), loc='lower right', borderaxespad=0, ncol=2, frameon=False)

            # Crop image
            plt.tight_layout()

            # Add footer
            cls._footer_bar(fig)

            # Save file
            output_filename = 'plot_data_send_yearly.png'
            output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
            fig.savefig(output_filepath, transparent=True)
            logger.info("Plot saved as %s" % output_filepath)

            return output_filename

    @classmethod
    def plot_data_send_monthly(cls, network, year=None, width=None, height=None, unit=None, colormap=None):
        """
        Plots the amount of data send monthly for a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param unit: The unit in which the values are converted
        :param colormap: The colormap to use
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_data_send_monthly(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, unit, colormap))
        year = year or datetime.datetime.now().year
        logger.info("Plotting data send monthly by network %s in %i" % (network.code, year))

        ureg = pint.UnitRegistry()
        ureg.setup_matplotlib(True)

        # Get plot data
        plot_data_months = []
        plot_data_dataselect = []
        plot_data_seedlink = []
        for month in range(1, 13):
            send_dataselect = network.data_send_dataselect(year=year, month=month) or 0
            send_seedlink = network.data_send_seedlink(year=year, month=month) or 0
            plot_data_dataselect.append(send_dataselect * ureg.byte)
            plot_data_seedlink.append(send_seedlink * ureg.byte)
            plot_data_months.append(arrow.get(year, month, 1).datetime)

        # Build image
        bar_width = 0.75 * ureg.month
        fig, ax = plt.subplots()

        # Calculate dimensions of image
        height = height or cls.DEFAULT_PX_HEIGHT
        width = width or cls.DEFAULT_PX_WIDTH
        fig.set_size_inches(width/fig.dpi, height/fig.dpi)

        # Initialize colors
        colors = cls._colors(colormap, 2)

        # Configure axis and plot data
        ax.yaxis.set_label_text('Data send (%s) for %s in %s' % (unit or 'Bytes', network.code, year))
        if unit:
            ax.yaxis.set_units(ureg.Unit(unit))
        ax.xaxis.set_tick_params(rotation=90)
        ax.xaxis.set_major_formatter(mdates.DateFormatter("%b"))
        ax.xaxis.set_major_locator(mdates.MonthLocator())
        ax.bar(plot_data_months, plot_data_seedlink, bar_width, label='Seedlink', color=colors[0])
        ax.bar(plot_data_months, plot_data_dataselect, bar_width, bottom=plot_data_seedlink, label='Dataselect', color=colors[1])

        # Add title
        #ax.set_title("%s %i" % (network.code, year), loc='left')

        # Add legend
        ax.legend(bbox_to_anchor=(1, 1), loc='lower right', borderaxespad=0, ncol=2, frameon=False)

        # Crop image
        plt.tight_layout()

        # Add footer
        cls._footer_bar(fig)

        # Save file
        output_filename = 'plot_data_send_monthly.png'
        output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
        fig.savefig(output_filepath, transparent=True)
        logger.info("Plot saved as %s" % output_filepath)

        return output_filename

    @classmethod
    def plot_data_send_by_other_networks(cls, network, year=None, width=None, height=None, unit=None):
        """
        Plots a pie chart with the amount of data send yearly for a network against the other networks
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param unit: The unit in which the values are converted
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_data_send_by_other_networks(%s, %s, %s, %s, %s)" % (network, year, width, height, unit))
        year = year or datetime.datetime.now().year
        logger.info("Plotting ratio of data send for network %s against all networks in %i" % (network.code, year))

        # Get plot data
        send_network = network.data_send(year=year)
        send_all = service_stats.data_send(year=year)
        plot_data_values = [send_network, send_all-send_network]
        plot_data_labels = [network.code, 'Other']
        plot_data_explode = [0.1, 0]

        # Build image
        fig, ax = plt.subplots()

        # Calculate dimensions of image
        height = height or cls.DEFAULT_PX_HEIGHT
        width = width or cls.DEFAULT_PX_WIDTH
        fig.set_size_inches(width/fig.dpi, height/fig.dpi)

        # Configure axis and plot data
        ax.pie(plot_data_values, labels=plot_data_labels, explode=plot_data_explode, startangle=90)
        ax.axis('equal')

        # Crop image
        plt.tight_layout()

        # Add footer
        cls._footer_bar(fig)

        # Save file
        output_filename = 'plot_data_send_by_other_networks.png'
        output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
        fig.savefig(output_filepath, transparent=True)
        logger.info("Plot saved as %s" % output_filepath)

        return output_filename

    @classmethod
    def plot_data_stored_yearly(cls, network, year=None, width=None, height=None, vertical=True, start_year=None, unit=None, colormap=None):
        """
        Plots the amount of data stored yearly for a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param vertical: Enables the vertical layout (default=false)
        :param start_year: The starting year
        :param unit: The unit in which the values are converted
        :param colormap: The colormap to use
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_data_stored_yearly(%s, %s, %s, %s, %s, %s, %s, %s)" % (network, year, width, height, vertical, start_year, unit, colormap))
        year = year or datetime.datetime.now().year
        logger.info("Plotting data stored yearly by network %s until %i" % (network.code, year))

        ureg = pint.UnitRegistry()
        ureg.setup_matplotlib(True)

        # Calculate starting year
        if start_year:
            start_year = max(network.year_start, start_year)
        else:
            start_year = network.year_start
        logger.debug('Starting year: %s' % start_year)

        # Get plot data
        plot_data_years = []
        plot_data_buffer = []
        plot_data_validated = []
        has_data = False
        for y in range(start_year, year+1):
            stored_buffer = network.data_stored_buffer(year=y)
            stored_validated = network.data_stored_validated(year=y)
            if has_data or stored_buffer or stored_validated:
                has_data = True
                plot_data_years.append(arrow.get(y, 1, 1).datetime)
                plot_data_buffer.append((stored_buffer or 0) * ureg.byte)
                plot_data_validated.append((stored_validated or 0) * ureg.byte)

        # Build image
        if plot_data_years:
            logger.debug("Plotted years: %s" % len(plot_data_years))
            bar_width = 0.75 * ureg.year
            fig, ax = plt.subplots()

            # Calculate dimensions of image
            bar_px = 50
            margin_px_top = 20
            margin_px_right = 10
            margin_px_left_bottom = 100
            if vertical:
                width_px = width or max(cls.MINIMAL_PX_WIDTH, (bar_px * len(plot_data_years) + margin_px_left_bottom + margin_px_right))
                height_px = height or cls.DEFAULT_PX_HEIGHT
            else:
                width_px = width or cls.DEFAULT_PX_WIDTH
                height_px = height or (bar_px * len(plot_data_years) + margin_px_top + margin_px_left_bottom)

            logger.debug("Figure size (Pixels): w=%s, h=%s" % (width_px, height_px))
            fig.set_size_inches(width_px/fig.dpi, height_px/fig.dpi)

            # Initialize colors
            colors = cls._colors(colormap, 2)

            # Configure axis and plot data
            ax.xaxis.set_tick_params(rotation=90)
            if vertical:
                ax.yaxis.set_label_text('Data stored (%s) for %s' % (unit or 'Bytes', network.code))
                if unit:
                    ax.yaxis.set_units(ureg.Unit(unit))
                ax.xaxis.set_major_formatter(mdates.DateFormatter("%Y"))
                ax.xaxis.set_major_locator(mdates.YearLocator())
                ax.bar(plot_data_years, plot_data_validated, bar_width, label='Validated', color=colors[0])
                ax.bar(plot_data_years, plot_data_buffer, bar_width, bottom=plot_data_validated, label='Non-validated', color=colors[1])
            else:
                ax.xaxis.set_label_text('Data stored (%s) for %s' % (unit or 'Bytes', network.code))
                if unit:
                    ax.xaxis.set_units(ureg.Unit(unit))
                ax.yaxis.set_major_formatter(mdates.DateFormatter("%Y"))
                ax.yaxis.set_major_locator(mdates.YearLocator())
                ax.barh(plot_data_years, plot_data_validated, bar_width, label='Validated', color=colors[0])
                ax.barh(plot_data_years, plot_data_buffer, bar_width, left=plot_data_validated, label='Non-validated', color=colors[1])

            # Add title
            #ax.set_title(network.code, loc='left')

            # Add legend
            ax.legend(bbox_to_anchor=(1, 1), loc='lower right', borderaxespad=0, ncol=2, frameon=False)

            # Crop image
            plt.tight_layout()

            # Add footer
            cls._footer_bar(fig)

            # Save file
            output_filename = 'plot_data_stored_yearly.png'
            output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
            fig.savefig(output_filepath, transparent=True)
            logger.info("Plot saved as %s" % output_filepath)

            return output_filename

    @classmethod
    def plot_requests_yearly(cls, network, year=None, width=None, height=None, vertical=True, start_year=None, colormap=None):
        """
        Plots the number of requests querying yearly a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param vertical: Enables the vertical layout (default=false)
        :param start_year: The starting year
        :param colormap: The colormap to use
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_requests_yearly(%s, %s, %s, %s, %s)" % (network, year, width, height, colormap))
        year = year or datetime.datetime.now().year
        logger.info("Plotting requests yearly by network %s until %i" % (network.code, year))

        ureg = pint.UnitRegistry()
        ureg.setup_matplotlib(True)

        # Calculate starting year
        if start_year:
            start_year = max(network.year_start, start_year)
        else:
            start_year = network.year_start
        logger.debug('Starting year: %s' % start_year)

        # Get plot data
        plot_data_years = []
        plot_data_dataselect = []
        plot_data_station = []
        has_data = False
        for y in range(start_year, year+1):
            requests_dataselect = network.requests_dataselect(year=y)
            requests_station = network.requests_station(year=y)
            if has_data or requests_dataselect or requests_station:
                has_data = True
                plot_data_years.append(arrow.get(y, 1, 1).datetime)
                plot_data_dataselect.append(requests_dataselect or 0)
                plot_data_station.append(requests_station or 0)

        # Build image
        if plot_data_years:
            logger.debug("Plotted years: %s" % len(plot_data_years))
            bar_width = 0.75 * ureg.year
            fig, ax = plt.subplots()

            # Calculate dimensions of image
            bar_px = 50
            margin_px_top = 20
            margin_px_right = 10
            margin_px_left_bottom = 100
            if vertical:
                width_px = width or max(cls.MINIMAL_PX_WIDTH, (bar_px * len(plot_data_years) + margin_px_left_bottom + margin_px_right))
                height_px = height or cls.DEFAULT_PX_HEIGHT
            else:
                width_px = width or cls.DEFAULT_PX_WIDTH
                height_px = height or (bar_px * len(plot_data_years) + margin_px_top + margin_px_left_bottom)

            logger.debug("Figure size (Pixels): w=%s, h=%s" % (width_px, height_px))
            fig.set_size_inches(width_px/fig.dpi, height_px/fig.dpi)

            # Initialize colors
            colors = cls._colors(colormap, 2)

            # Configure axis and plot data
            ax.xaxis.set_tick_params(rotation=90)
            if vertical:
                ax.yaxis.set_label_text('Requests for %s' % network.code)
                ax.xaxis.set_major_formatter(mdates.DateFormatter("%Y"))
                ax.xaxis.set_major_locator(mdates.YearLocator())
                ax.bar(plot_data_years, plot_data_station, bar_width, label='Station', color=colors[0])
                ax.bar(plot_data_years, plot_data_dataselect, bar_width, bottom=plot_data_station, label='Dataselect', color=colors[1])
            else:
                ax.xaxis.set_label_text('Requests for %s' % network.code)
                ax.yaxis.set_major_formatter(mdates.DateFormatter("%Y"))
                ax.yaxis.set_major_locator(mdates.YearLocator())
                ax.barh(plot_data_years, plot_data_station, bar_width, label='Station', color=colors[0])
                ax.barh(plot_data_years, plot_data_dataselect, bar_width, left=plot_data_station, label='Dataselect', color=colors[1])

            # Add title
            #ax.set_title(network.code, loc='left')

            # Add legend
            ax.legend(bbox_to_anchor=(1, 1), loc='lower right', borderaxespad=0, ncol=2, frameon=False)

            # Crop image
            plt.tight_layout()

            # Add footer
            cls._footer_bar(fig)

            # Save file
            output_filename = 'plot_requests_yearly.png'
            output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
            fig.savefig(output_filepath, transparent=True)
            logger.info("Plot saved as %s" % output_filepath)

            return output_filename

    @classmethod
    def plot_requests_monthly(cls, network, year=None, width=None, height=None, colormap=None):
        """
        Plots the number of requests querying monthly a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param colormap: The colormap to use
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_requests_monthly(%s, %s, %s, %s, %s)" % (network, year, width, height, colormap))
        year = year or datetime.datetime.now().year
        logger.info("Plotting requests monthly by network %s in %i" % (network.code, year))

        ureg = pint.UnitRegistry()
        ureg.setup_matplotlib(True)

        # Get plot data
        plot_data_months = []
        plot_data_dataselect = []
        plot_data_station = []
        for month in range(1, 13):
            requests_dataselect = network.requests_dataselect(year=year, month=month) or 0
            requests_station = network.requests_station(year=year, month=month) or 0

            plot_data_dataselect.append(requests_dataselect)
            plot_data_station.append(requests_station)
            plot_data_months.append(arrow.get(year, month, 1).datetime)

        # Build image
        bar_width = 0.75 * ureg.month
        fig, ax = plt.subplots()

        # Calculate dimensions of image
        height = height or cls.DEFAULT_PX_HEIGHT
        width = width or cls.DEFAULT_PX_WIDTH
        fig.set_size_inches(width/fig.dpi, height/fig.dpi)

        # Initialize colors
        colors = cls._colors(colormap, 2)

        # Configure axis and plot data
        ax.yaxis.set_label_text('Requests for %s in %s' % (network.code, year))
        ax.xaxis.set_tick_params(rotation=90)
        ax.xaxis.set_major_formatter(mdates.DateFormatter("%b"))
        ax.xaxis.set_major_locator(mdates.MonthLocator())
        ax.bar(plot_data_months, plot_data_station, bar_width, label='Station', color=colors[0])
        ax.bar(plot_data_months, plot_data_dataselect, bar_width, bottom=plot_data_station, label='Dataselect', color=colors[1])

        # Add title
        #ax.set_title("%s %i" % (network.code, year), loc='left')

        # Add legend
        ax.legend(bbox_to_anchor=(1, 1), loc='lower right', borderaxespad=0, ncol=2, frameon=False)

        # Crop image
        plt.tight_layout()

        # Add footer
        cls._footer_bar(fig)

        # Save file
        output_filename = 'plot_requests_monthly.png'
        output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
        fig.savefig(output_filepath, transparent=True)
        logger.info("Plot saved as %s" % output_filepath)

        return output_filename

    @classmethod
    def plot_map_requests_by_country(cls, network, year=None, width=None, height=None, colormap='Blues', dpi=100):
        """
        Plots a world map colored by number of requests querying a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param colormap: The colormap to use
        :param dpi: The DPI (default=100)
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_map_requests_by_country(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, colormap, dpi))
        year = year or datetime.datetime.now().year
        logger.info("Plotting map of requests by country for network %s in %i" % (network.code, year))

        # Build plot data
        plot_data = {}
        with_antartica = False
        for country_code, country_requests in network.requests_by_country(year=year).items():
            try:
                country = countries.get(country_code)
                plot_data[country.alpha3] = int(country_requests)
                if not with_antartica and country.alpha3 == 'ATA':
                    with_antartica = True
            except KeyError:
                logger.debug("Unable to resolve country code '%s'" % country_code)
                continue

        # Build output
        if plot_data:

            # Initialize figure
            height = height or cls.DEFAULT_PX_HEIGHT
            width = width or cls.DEFAULT_PX_WIDTH
            fig = plt.figure(figsize=(width/dpi, height/dpi))
            ax = fig.add_subplot(1, 1, 1)
            ax.tick_params(axis='both', bottom=False, left=False,  labelbottom=False, labelleft=False)

            # Plot data
            df = pd.DataFrame({
                'iso_a3': pd.Series(plot_data.keys()),
                'requests': pd.Series(plot_data.values())
            })
            world = geopandas.read_file(
                geopandas.datasets.get_path("naturalearth_lowres"),
                ignore_fields=["pop_est", "continent", "gdp_md_est"]
            )
            world = world.merge(df, on='iso_a3', how='left')

            if not with_antartica:
                world.drop(world.loc[world['iso_a3']=='ATA'].index, inplace=True)

            world.plot("requests", ax=ax, cmap=colormap, edgecolor="grey",
                       legend=True, legend_kwds={"label":"Requests", 'orientation':"horizontal"},
                       missing_kwds={"color": "lightgrey", "label":"Missing Values", 'edgecolor':None}
            )

            # Add title
            # ax.set_title("Requests for %s in %s" % (network.code, year))

            # Crop image
            #plt.tight_layout()

            # Add footer
            cls._footer_bar(fig)

            # Save file
            output_filename = 'plot_map_requests_by_country.png'
            output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
            fig.savefig(output_filepath, transparent=True)
            logger.info("Plot saved as %s" % output_filepath)

            return output_filename

    @classmethod
    def plot_map_clients_by_country(cls, network, year=None, width=None, height=None, colormap='Blues', dpi=100):
        """
        Plots a world map colored by number of clients querying a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param colormap: The colormap to use
        :param dpi: The DPI (default=100)
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_map_clients_by_country(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, colormap, dpi))
        year = year or datetime.datetime.now().year
        logger.info("Plotting map of clients by country for network %s in %i" % (network.code, year))

        # Build plot data
        plot_data = {}
        with_antartica = False
        for country_code, country_clients in network.clients_by_country(year=year).items():
            try:
                country = countries.get(country_code)
                plot_data[country.alpha3] = int(country_clients)
                if not with_antartica and country.alpha3 == 'ATA':
                    with_antartica = True
            except KeyError:
                logger.debug("Unable to resolve country code '%s'" % country_code)
                continue

        # Build output
        if plot_data:

            # Initialize figure
            height = height or cls.DEFAULT_PX_HEIGHT
            width = width or cls.DEFAULT_PX_WIDTH
            fig = plt.figure(figsize=(width/dpi, height/dpi))
            ax = fig.add_subplot(1, 1, 1)
            ax.tick_params(axis='both', bottom=False, left=False,  labelbottom=False, labelleft=False)

            # Plot data
            df = pd.DataFrame({
                'iso_a3': pd.Series(plot_data.keys()),
                'clients': pd.Series(plot_data.values())
            })
            world = geopandas.read_file(
                geopandas.datasets.get_path("naturalearth_lowres"),
                ignore_fields=["pop_est", "continent", "gdp_md_est"]
            )
            world = world.merge(df, on='iso_a3', how='left')

            if not with_antartica:
                world.drop(world.loc[world['iso_a3']=='ATA'].index, inplace=True)

            world.plot("clients", ax=ax, cmap=colormap, edgecolor="grey",
                       legend=True, legend_kwds={"label":"Clients", 'orientation':"horizontal"},
                       missing_kwds={"color": "lightgrey", "label":"Missing Values", 'edgecolor':None}
            )

            # Add title
            # ax.set_title("clients for %s in %s" % (network.code, year))

            # Crop image
            #plt.tight_layout()

            # Add footer
            cls._footer_bar(fig)

            # Save file
            output_filename = 'plot_map_clients_by_country.png'
            output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
            fig.savefig(output_filepath, transparent=True)
            logger.info("Plot saved as %s" % output_filepath)

            return output_filename

    @classmethod
    def plot_map_network_stations(cls, network, year=None, width=None, height=None, zoom=None):
        """
        Plots a world map locating the stations of a network
        :param network: The network object
        :param year: The year
        :param width: The width of the output image
        :param height: The height of the output image
        :param zoom: The map zoom
        :return: The plot output file path
        :rtype: str
        """
        logger.debug("ServicePlotterPNG.plot_map_network_stations(%s, %s, %s, %s, %s)" % (network, year, width, height, zoom))
        year = year or datetime.datetime.now().year
        logger.info("Plotting map of stations for network %s in %i" % (network.code, year))

        root_path = pathlib.Path(__file__).resolve().parent.parent.parent
        marker_filepath = os.path.join(root_path, 'assets', 'triangle-10.png')

        width = width or cls.DEFAULT_PX_WIDTH
        height = height or cls.DEFAULT_PX_HEIGHT
        map = StaticMap(width, height)
        for station in network.active_stations(year):
            marker = IconMarker((station.longitude, station.latitude), marker_filepath, 0, 0)
            map.add_marker(marker)

        output_filename = 'plot_map_network_stations.png'
        output_filepath = os.path.join(cls.output_dir(network, year), output_filename)

        image = map.render(zoom=zoom)
        image.save(output_filepath)
        logger.info("Plot saved as %s" % output_filepath)

        return output_filename



# encoding: utf8
from ordered_enum import OrderedEnum
from enum import Enum


class EnumChannelBand(OrderedEnum):
    F = 'F'
    G = 'G'
    D = 'D'
    C = 'C'
    E = 'E'
    S = 'S'
    H = 'H'
    B = 'B'
    M = 'M'
    L = 'L'
    V = 'V'
    U = 'U'
    R = 'R'
    P = 'P'
    T = 'T'
    Q = 'Q'
    A = 'A'
    O = 'O'


class EnumChannelInstrument(Enum):
    H = 'H'  # High gain seismometer
    L = 'L'  # Low gain seismometer
    G = 'G'  # Gravimeter
    M = 'M'  # Mass position seismometer
    N = 'N'  # Accelerometer
    A = 'A'  # Tilt meter
    B = 'B'  # Creep meter
    C = 'C'  # Calibration input
    D = 'D'  # Pressure
    E = 'E'  # Electronic test point
    F = 'F'  # Magnetometer
    I = 'I'  # Humidity
    J = 'J'  # High gain seismometer
    K = 'K'  # Temperature
    O = 'O'  # Water current
    P = 'P'  # Geophone
    Q = 'Q'  # Electric potential
    R = 'R'  # Rainfall
    S = 'S'  # Linear strain
    T = 'T'  # Tide
    U = 'U'  # Bolometer
    V = 'V'  # Volumetric strain
    W = 'W'  # Wind
    X = 'X'  # Derived or generated channel
    Y = 'Y'  # Non-specific instrument
    Z = 'Z'  # Synthesized beams



# # encoding: utf8
# import logging
# import os
# import datetime
# import folium
# import statistics
# import plotly.express as px
# import plotly.graph_objects as go
# from iso3166 import countries
# from io import StringIO
# from resif_delivery_stats_plotter.services.clients.availabilty import ServiceClientAvailability as service_availability
# from . import ServicePlotterAbstract
#
# logger = logging.getLogger(__name__)
#
#
# class ServicePlotterHTML(ServicePlotterAbstract):
#     """
#     Dynamic JS plotter
#     """
#
#     FORMAT_FULL_HTML = 'full_html'
#     FORMAT_DIV = 'div'
#
#     @classmethod
#     def plot_network_availability(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML):
#         """
#         Plots the annual availability of each station of a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_network_availability(%s, %s, %s, %s, %s)" % (network, year, width, height, output_format))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting availability of network %s in %i" % (network.code, year))
#
#         # Initialize colors and labels
#         qualities = {}
#         qualities_colors_map = {
#             'Valid': 'blue',
#             'Raw': 'orange',
#         }
#         for quality in (service_availability.QUALITY_Q_CONTROLLED, service_availability.QUALITY_M_MODIFIED):
#             qualities[quality] = 'Valid'
#         for quality in (service_availability.QUALITY_R_RAW, service_availability.QUALITY_D_INDETERMINATE):
#             qualities[quality] = 'Raw'
#
#         # Build plot data
#         plot_data = []
#         for station in network.stations:
#             for channel in station.representative_channels(year):
#                 channel_qualities = service_availability.channel(channel.code, channel.location, station.code, network.code, year)
#                 if channel_qualities:
#                     for quality, timespans in channel_qualities.items():
#                         for timespan in timespans:
#                             plot_data.append(
#                                 dict(Station="%s.%s.%s" % (network.code, station.code, channel.location_code),
#                                      Start=timespan[0], Finish=timespan[1], Quality=qualities.get(quality))
#                                 )
#
#         # Save
#         figure = px.timeline(plot_data, x_start="Start", x_end="Finish", y="Station", color="Quality",
#                              color_discrete_map=qualities_colors_map)
#
#         if output_format == cls.FORMAT_DIV:
#             with StringIO() as buffer:
#                 figure.write_html(
#                     include_plotlyjs=False,
#                     full_html=False,
#                     file=buffer
#                 )
#                 output = buffer.getvalue()
#
#             logger.info("Plot build as a <div>")
#             return output
#
#         else:
#             output_filename = 'plot_data_availability_network.html'
#             output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
#             figure.write_html(
#                 include_plotlyjs='cdn',
#                 full_html=True,
#                 file=output_filepath
#             )
#             logger.info("Plot saved as %s" % output_filepath)
#             return output_filename
#
#     @classmethod
#     def plot_data_send_yearly(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML, start_year=None, unit=None):
#         """
#         Plots the amount of data send yearly for a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :param start_year: The starting year
#         :param unit: The unit in which the values are converted
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_data_send_yearly(%s, %s, %s, %s, %s, %s, %s)" % (network, year, width, height, output_format, start_year, unit))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting data send yearly by network %s until %i" % (network.code, year))
#
#         #Build plot data
#         plot_data_years = []
#         plot_data_dataselect = []
#         plot_data_seedlink = []
#
#         # Calculate starting year
#         if start_year:
#             start_year = max(start_year, start_year)
#         else:
#             start_year = network.year_start
#         logger.debug('Starting year: %s' % start_year)
#
#         # Plot data
#         has_data = False
#         for y in range(start_year, year+1):
#             send_dataselect = network.data_send_dataselect(year=y, unit=unit)
#             send_seedlink = network.data_send_seedlink(year=y, unit=unit)
#             if has_data or send_dataselect or send_seedlink:
#                 has_data = True
#                 plot_data_years.append(y)
#                 plot_data_dataselect.append(send_dataselect or 0)
#                 plot_data_seedlink.append(send_seedlink or 0)
#
#         # Save
#         figure = go.Figure(data=[
#             go.Bar(name='Seedlink (%s)' % (unit or 'Bytes'), x=plot_data_years, y=plot_data_seedlink),
#             go.Bar(name='Dataselect (%s)' % (unit or 'Bytes'), x=plot_data_years, y=plot_data_dataselect)
#         ])
#         figure.update_layout(barmode='stack')
#
#         if output_format == cls.FORMAT_DIV:
#             with StringIO() as buffer:
#                 figure.write_html(
#                     include_plotlyjs=False,
#                     full_html=False,
#                     file=buffer
#                 )
#                 output = buffer.getvalue()
#
#             logger.info("Plot build as a <div>")
#             return output
#
#         else:
#             output_filename = 'plot_data_send_yearly.html'
#             output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
#             figure.write_html(
#                 include_plotlyjs='cdn',
#                 full_html=True,
#                 file=output_filepath
#             )
#             logger.info("Plot saved as %s" % output_filepath)
#             return output_filename
#
#     @classmethod
#     def plot_data_send_monthly(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML, unit=None):
#         """
#         Plots the amount of data send monthly for a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :param unit: The unit in which the values are converted
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_data_send_monthly(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, output_format, unit))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting data send monthly by network %s in %i" % (network.code, year))
#
#         # Build plot data
#         plot_data_months = []
#         plot_data_dataselect = []
#         plot_data_seedlink = []
#
#         for month in range(1, 13):
#             send_dataselect = network.data_send_dataselect(year=year, month=month, unit=unit) or 0
#             send_seedlink = network.data_send_seedlink(year=year, month=month, unit=unit) or 0
#
#             plot_data_dataselect.append(send_dataselect)
#             plot_data_seedlink.append(send_seedlink)
#             plot_data_months.append(month)
#
#         # Build HTML output
#         figure = go.Figure(data=[
#             go.Bar(name='Seedlink (%s)' % (unit or 'Bytes'), x=plot_data_months, y=plot_data_seedlink),
#             go.Bar(name='Dataselect (%s)' % (unit or 'Bytes'), x=plot_data_months, y=plot_data_dataselect)
#         ])
#         figure.update_layout(barmode='stack')
#
#         if output_format == cls.FORMAT_DIV:
#             with StringIO() as buffer:
#                 figure.write_html(
#                     include_plotlyjs=False,
#                     full_html=False,
#                     file=buffer
#                 )
#                 output = buffer.getvalue()
#
#             logger.info("Plot build as a <div>")
#             return output
#
#         else:
#             output_filename = 'plot_data_send_monthly.html'
#             output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
#             figure.write_html(
#                 include_plotlyjs='cdn',
#                 full_html=True,
#                 file=output_filepath
#             )
#             logger.info("Plot saved as %s" % output_filepath)
#             return output_filename
#
#     @classmethod
#     def plot_data_send_by_other_networks(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML, unit=None):
#         """
#         Plots a pie chart with the amount of data send yearly for a network against the other networks
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :param unit: The unit in which the values are converted
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_data_send_by_other_networks(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, output_format, unit))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting ratio of data send for network %s against all networks in %i" % (network.code, year))
#         # TODO
#         pass
#
#     @classmethod
#     def plot_data_stored_yearly(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML, start_year=None, unit=None):
#         """
#         Plots the amount of data stored yearly for a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :param start_year: The starting year
#         :param unit: The unit in which the values are converted
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_data_stored_yearly(%s, %s, %s, %s, %s, %s, %s)" % (network, year, width, height, output_format, start_year, unit))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting data stored yearly by network %s until %i" % (network.code, year))
#         # TODO
#         pass
#
#     @classmethod
#     def plot_map_requests_by_country(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML):
#         """
#         Plots a world map colored by number of requests querying a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_map_requests_by_country(%s, %s, %s, %s, %s)" % (network, year, width, height, output_format))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting map of requests by country for network %s in %i" % (network.code, year))
#
#         # Build plot data
#         plot_data_locations = []
#         plot_data_values = []
#
#         for country_code, country_requests in network.requests_by_country(year=year).items():
#             try:
#                 country = countries.get(country_code)
#                 plot_data_locations.append(country.alpha3)
#                 plot_data_values.append(int(country_requests))
#             except KeyError:
#                 logger.debug("Unable to resolve country code '%s'" % country_code)
#                 continue
#
#         # Output
#         figure = go.Figure(data=go.Choropleth(locations=plot_data_locations, z=plot_data_values, colorbar_title='Requests', colorscale='Reds'))
#
#         if output_format == cls.FORMAT_DIV:
#             with StringIO() as buffer:
#                 figure.write_html(
#                     include_plotlyjs=False,
#                     full_html=False,
#                     file=buffer
#                 )
#                 output = buffer.getvalue()
#
#             logger.info("Plot build as a <div>")
#             return output
#
#         else:
#             output_filename = 'plot_map_requests_by_country.html'
#             output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
#             figure.write_html(
#                 include_plotlyjs='cdn',
#                 full_html=True,
#                 file=output_filepath
#             )
#             logger.info("Plot saved as %s" % output_filepath)
#             return output_filename
#
#     @classmethod
#     def plot_map_clients_by_country(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML):
#         """
#         Plots a world map colored by number of clients querying a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_map_clients_by_country(%s, %s, %s, %s, %s)" % (network, year, width, height, output_format))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting map of clients by country for network %s in %i" % (network.code, year))
#
#         # Build plot data
#         plot_data_locations = []
#         plot_data_values = []
#
#         for country_code, country_requests in network.clients_by_country(year=year).items():
#             try:
#                 country = countries.get(country_code)
#                 plot_data_locations.append(country.alpha3)
#                 plot_data_values.append(int(country_requests))
#             except KeyError:
#                 logger.debug("Unable to resolve country code '%s'" % country_code)
#                 continue
#
#         # Output
#         figure = go.Figure(data=go.Choropleth(locations=plot_data_locations, z=plot_data_values, colorbar_title='Requests', colorscale='Reds'))
#
#         if output_format == cls.FORMAT_DIV:
#             with StringIO() as buffer:
#                 figure.write_html(
#                     include_plotlyjs=False,
#                     full_html=False,
#                     file=buffer
#                 )
#                 output = buffer.getvalue()
#
#             logger.info("Plot build as a <div>")
#             return output
#
#         else:
#             output_filename = 'plot_map_clients_by_country.html'
#             output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
#             figure.write_html(
#                 include_plotlyjs='cdn',
#                 full_html=True,
#                 file=output_filepath
#             )
#             logger.info("Plot saved as %s" % output_filepath)
#             return output_filename
#
#     @classmethod
#     def plot_map_network_stations(cls, network, year=None, width=None, height=None, output_format=FORMAT_FULL_HTML, zoom=None):
#         """
#         Plots a world map locating the stations of a network
#         :param network: The network object
#         :param year: The year
#         :param width: The width of the output image
#         :param height: The height of the output image
#         :param output_format: The output format (default='full_html' | 'div')
#         :param zoom: The map zoom
#         :return: The plot output file path or the div content
#         :rtype: str
#         """
#         logger.debug("ServicePlotterHTML.plot_map_network_stations(%s, %s, %s, %s, %s, %s)" % (network, year, width, height, output_format, zoom))
#         year = year or datetime.datetime.now().year
#         logger.info("Plotting map of stations for network %s in %i" % (network.code, year))
#
#         folium_markers = []
#         latitudes = []
#         longitudes = []
#
#         for station in network.stations:
#             latitudes.append(station.latitude)
#             longitudes.append(station.longitude)
#             folium_markers.append(
#                 folium.Marker(
#                     location=[station.latitude, station.longitude],
#                     popup=station.code,
#                 )
#             )
#
#         folium_map = folium.Map(
#             location=[statistics.mean(latitudes), statistics.mean(longitudes)],
#             zoom_start=5
#         )
#
#         for folium_marker in folium_markers:
#             folium_marker.add_to(folium_map)
#
#         # Output
#         if output_format == cls.FORMAT_DIV:
#             logger.info("Plot build as a <div>")
#             return folium_map._repr_html_()
#
#         else:
#             output_filename = 'plot_map_network_stations.html'
#             output_filepath = os.path.join(cls.output_dir(network, year), output_filename)
#             folium_map.save(output_filepath)
#             logger.info("Plot saved as %s" % output_filepath)
#             return output_filename



#!/usr/bin/env python
# encoding: utf8
import click
import click_log
import logging
import requests_cache
from .errors import NoDataError, ApiError
from .factories.network import NetworkFactory
from .factories.plotter import ServicePlotterFactory
from .factories.report import ReportFactory

# Configure logger
logger = logging.getLogger('resif_delivery_stats_plotter')
click_log.basic_config(logger)

# Configure requests cache (expire after 1h)
requests_cache.install_cache('cache', expire_after=3600)


@click.group()
def cli():
    pass


# REPORT
########################################################################################################################

@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'report_format', default='png')
@click.option('--unit', default='GB')
@click.option('--no-plots', 'no_plots', is_flag=True)
def report(network, year, width, height, report_format, unit, no_plots):
    """Generates an annual report of a network"""

    click.echo("Building %i annual report for '%s' network" % (year, network))
    try:
        report = ReportFactory.factory(year, network)
        report.build(output_format=report_format, unit=unit, width=width, height=height, with_plots=(not no_plots))
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


# PLOTS
########################################################################################################################

@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
def plot_network_availability(network_code, year, width, height, image_format):
    """Plots the annual availability of each station of a network"""

    click.echo("Plotting the %i availability of '%s' network" % (year, network_code))
    try:
        network = NetworkFactory.factory(network_code, channel_year=year)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_network_availability(network=network, year=year, width=width, height=height)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
@click.option('--unit', default='GB')
def plot_data_send_yearly(network_code, year, width, height, image_format, unit):
    """Plots the amount of data send yearly for a network"""

    click.echo("Plotting the yearly amount data send for '%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_data_send_yearly(network=network, year=year, width=width, height=height, unit=unit)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
@click.option('--unit', default='GB')
def plot_data_send_monthly(network_code, year, width, height, image_format, unit):
    """Plots the amount of data send monthly for a network"""

    click.echo("Plotting the monthly amount data send for '%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_data_send_monthly(network=network, year=year, width=width, height=height, unit=unit)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


# @cli.command()
# @click_log.simple_verbosity_option(logger)
# @click.argument('network_code', type=click.STRING)
# @click.argument('year', type=click.INT)
# @click.option('--width', type=click.INT)
# @click.option('--height', type=click.INT)
# @click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
# @click.option('--unit', default='GB')
# def plot_data_send_against_other_networks(network_code, year, width, height, image_format, unit):
#     """Plots a pie chart of the amount of data send for a network against other networks"""
#
#     click.echo("Plotting a pie chart of the amount of data send for '%s' network against other networks" % network_code)
#     try:
#         network = NetworkFactory.factory(network_code)
#         service_plotter = ServicePlotterFactory.factory(image_format)
#         service_plotter.plot_data_send_by_other_networks(network=network, year=year, width=width, height=height, unit=unit)
#     except NoDataError:
#         click.echo("No data found!")
#     except ApiError as e:
#         click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
@click.option('--unit', default='GB')
def plot_data_stored_yearly(network_code, year, width, height, image_format, unit):
    """Plots the amount of data send stored for a network"""

    click.echo("Plotting the monthly amount data stored for '%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_data_stored_yearly(network=network, year=year, width=width, height=height, unit=unit)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
def plot_requests_yearly(network_code, year, width, height, image_format):
    """Plots the number of requests querying yearly a network"""

    click.echo("Plotting the number of requests querying yearly the'%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_requests_yearly(network=network, year=year, width=width, height=height)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
def plot_requests_monthly(network_code, year, width, height, image_format):
    """Plots the number of requests querying yearly a network"""

    click.echo("Plotting the number of requests querying monthly the'%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_requests_monthly(network=network, year=year, width=width, height=height)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
def plot_map_requests_by_country(network_code, year, width, height, image_format):
    """Plots on a world map the number of requests querying a network"""

    click.echo("Plotting on a world map the number of requests querying the'%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_map_requests_by_country(network=network, year=year, width=width, height=height)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
def plot_map_clients_by_country(network_code, year, width, height, image_format):
    """Plots on a world map the number of clients querying a network"""

    click.echo("Plotting on a world map the number of clients querying the'%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_map_clients_by_country(network=network, year=year, width=width, height=height)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('year', type=click.INT)
@click.option('--width', type=click.INT)
@click.option('--height', type=click.INT)
@click.option('--zoom', type=click.INT)
@click.option('--format', 'image_format', default=ServicePlotterFactory.FORMAT_PNG)
def plot_map_network_stations(network_code, year, width, height, zoom, image_format):
    """Plots on a world map the stations a network"""

    click.echo("Plotting on a map locating the stations of the'%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        service_plotter = ServicePlotterFactory.factory(image_format)
        service_plotter.plot_map_network_stations(network=network, year=year, width=width, height=height, zoom=zoom)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


# CACHE
########################################################################################################################

@cli.command()
@click_log.simple_verbosity_option(logger)
def clear_requests_cache():
    requests_cache.clear()
    click.echo("Requests cache cleared!")


# DEBUG
########################################################################################################################


@cli.command()
@click_log.simple_verbosity_option(logger)
def list_networks():
    """Lists the networks"""

    click.echo("Listing the networks")
    try:
        networks = NetworkFactory.list()
        for network in networks:
            click.echo(network)
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.option('--year', type=click.INT)
def list_stations(network_code, year):
    """Lists the stations of a network"""

    click.echo("Listing the stations of the '%s' network" % network_code)
    try:
        network = NetworkFactory.factory(network_code)
        for station in network.stations:
            click.echo(station)
            if year:
                click.echo(' > Open in %s: %s' % (year, station.is_open(year)))
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


@cli.command()
@click_log.simple_verbosity_option(logger)
@click.argument('network_code', type=click.STRING)
@click.argument('station_code', type=click.STRING)
@click.option('--year', type=click.INT)
def list_channels(network_code, station_code, year):
    """Lists the channels of a station of network"""

    click.echo("Listing the channels of the '%s' station from the network" % (station_code, network_code))
    try:
        network = NetworkFactory.factory(network_code)
        station = network.get_station(station_code)
        for channel in station.channels:
            click.echo(channel)
            if year:
                click.echo(' > Open in %s: %s' % (year, channel.is_open(year)))
    except NoDataError:
        click.echo("No data found!")
    except ApiError as e:
        click.echo(e)


if __name__ == '__main__':
    cli()



# encoding: utf8
import re
regex_location_code = re.compile("(?:(?P<location>.*)\.)?(?P<band>\w)(?P<instrument>\w)(?P<orientation>\w)")


class LocationCodeMixin:

    @staticmethod
    def extract_location_from_location_code(location_code):
        """
        Extracts the band code from a channel location code

        :param location_code: The channel location code
        :return: The location
        :rtype: str
        """
        matches = regex_location_code.search(location_code)
        return matches.group('location') or None

    @staticmethod
    def extract_band_from_location_code(location_code):
        """
        Extracts the band code from a channel location code

        :param location_code: The channel location code
        :return: The band code
        :rtype: str
        """
        matches = regex_location_code.search(location_code)
        return matches.group('band')

    @staticmethod
    def extract_instrument_from_location_code(location_code):
        """
        Extracts the instrument code from a channel location code

        :param location_code: The channel location code
        :return: The instrument code
        :rtype: str
        """
        matches = regex_location_code.search(location_code)
        return matches.group('instrument')

    @staticmethod
    def extract_orientation_from_location_code(location_code):
        """
        Extracts the orientation code from a channel location code

        :param location_code: The channel location code
        :return: The orientation code
        :rtype: str
        """
        matches = regex_location_code.search(location_code)
        return matches.group('orientation')


# encoding: utf8
from resif_delivery_stats_plotter.services.clients.statistics import ServiceClientStatistics as service_stats


class Network:
    """
    Defines a network
    """

    def __init__(self, code=None, start_date=None, end_date=None, stations=None):
        self.code = code
        self._stations = stations or {}

        if isinstance(start_date, tuple):
            self._start_date = start_date[0]
        else:
            self._start_date = start_date

        if isinstance(end_date, tuple):
            self._end_date = end_date[0]
        else:
            self._end_date = end_date

    def __repr__(self):
        return "<Network %s (%s | %s)>" % (
            self.code,
            self.date_start, self.date_end,
        )

    @property
    def date_start(self):
        """
        Gets the open date in ISO format for the current network

        :rtype str
        """
        if self._start_date:
            return self._start_date.isoformat()

    @property
    def date_end(self):
        """
        Gets the close date in ISO format for the current network

        :rtype str
        """
        if self._end_date:
            return self._end_date.isoformat()

    @property
    def year_start(self):
        """
        Gets the open year for the current network

        :rtype str
        """
        if self._start_date:
            return self._start_date.year

    # STATIONS
    ####################################################################################################################

    def add_station(self, station):
        """
        Adds a station to the current network

        :param station: The station to add
        """
        if station is not None:
            station._network_code = self.code
            self._stations[station.code] = station

    def get_station(self, code):
        """
        Gets a station by its code from the current network

        :param code: The code of the station to get
        :return: A station
        :rtype: Station
        """
        return self._stations.get(code)

    @property
    def stations(self):
        """
        Gets the list of stations of the current network

        :return: A collection of stations
        :rtype: list
        """
        return sorted(self._stations.values())

    def active_stations(self, year):
        """
        Gets the list of active stations of the current network

        :return: A collection of stations
        :rtype: list
        """
        stations = []
        for station in self.stations:
            if station.is_open(year):
                stations.append(station)
        return stations

    # STATISTICS
    ####################################################################################################################

    def requests(self, year=None, month=None, country=None, media=None):
        """
        Get the requests number for the current network

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :param media:  The media to filter the requests
        :return: The requests number
        :rtype: int
        """
        return service_stats.requests(network=self.code, year=year, month=month, country=country, media=media)

    def requests_seedlink(self, year=None, month=None, country=None):
        """
        Get the requests number querying seedlink for the current network

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :return: The requests number
        :rtype: int
        """
        return self.requests(year=year, month=month, country=country, media='seedlink')

    def requests_dataselect(self, year=None, month=None, country=None):
        """
        Get the requests number querying dataselect for the current network

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :return: The requests number
        :rtype: int
        """
        return self.requests(year=year, month=month, country=country, media='dataselect')

    def requests_station(self, year=None, month=None, country=None):
        """
        Get the requests number querying station for the current network

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :return: The requests number
        :rtype: int
        """
        return self.requests(year=year, month=month, country=country, media='station')

    def requests_by_country(self, year=None, month=None, country=None, media=None):
        """
        Get the requests number for each country, for the current network

        :param year: The year to filter the requests
        :param month:  The month to filter the requests
        :param country:  The country to filter the requests
        :param media:  The media to filter the requests
        :return: The requests number
        :rtype: int
        """
        return service_stats.requests_by_country(network=self.code, year=year, month=month, country=country, media=media)

    def requests_seedlink_by_country(self, year=None, month=None, country=None):
        """
        Get the requests number for each country, querying seedlink for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :return: The clients number
        :rtype: int
        """
        return self.requests_by_country(year=year, month=month, country=country, media='seedlink')

    def requests_dataselect_by_country(self, year=None, month=None, country=None):
        """
        Get the requests number for each country, querying dataselect for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :return: The clients number
        :rtype: int
        """
        return self.requests_by_country(year=year, month=month, country=country, media='dataselect')

    def requests_station_by_country(self, year=None, month=None, country=None):
        """
        Get the requests number for each country, querying station for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :return: The clients number
        :rtype: int
        """
        return self.requests_by_country(year=year, month=month, country=country, media='station')

    def clients(self, year=None, month=None, country=None, media=None):
        """
        Get the clients number for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :param media:  The media to filter the clients
        :return: The clients number
        :rtype: int
        """
        return service_stats.clients(network=self.code, year=year, month=month, country=country, media=media)

    def clients_by_country(self, year=None, month=None, country=None, media=None):
        """
        Get the clients number for each country, for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :param media:  The media to filter the clients
        :return: The clients number
        :rtype: int
        """
        return service_stats.clients_by_country(network=self.code, year=year, month=month, country=country, media=media)

    def clients_seedlink_by_country(self, year=None, month=None, country=None):
        """
        Get the clients number for each country, requesting seedlink for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :return: The clients number
        :rtype: int
        """
        return self.clients_by_country(year=year, month=month, country=country, media='seedlink')

    def clients_dataselect_by_country(self, year=None, month=None, country=None):
        """
        Get the clients number for each country, requesting dataselect for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :return: The clients number
        :rtype: int
        """
        return self.clients_by_country(year=year, month=month, country=country, media='dataselect')

    def clients_station_by_country(self, year=None, month=None, country=None):
        """
        Get the clients number for each country, requesting station for the current network

        :param year: The year to filter the clients
        :param month:  The month to filter the clients
        :param country:  The country to filter the clients
        :return: The clients number
        :rtype: int
        """
        return self.clients_by_country(year=year, month=month, country=country, media='station')

    def data_send(self, year=None, month=None, country=None, media=None, unit=None):
        """
        Gets the amount of data send for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param media:  The media to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send
        :rtype: int
        """
        return service_stats.data_send(network=self.code, year=year, month=month, country=country, media=media, unit=unit)

    def data_send_dataselect(self, year=None, month=None, country=None, unit=None):
        """
        Gets the amount of data send by dataselect for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by dataselect
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='dataselect', unit=unit)

    def data_send_seedlink(self, year, month=None, country=None, unit=None):
        """
        Gets the amount of data send by seedlink for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by seedlink
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='seedlink', unit=unit)

    def data_send_station(self, year, month=None, country=None, unit=None):
        """
        Gets the amount of data send by station WS for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param country:  The country to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send by station WS
        :rtype: int
        """
        return self.data_send(year=year, month=month, country=country, media='station', unit=unit)

    def data_stored(self, year=None, month=None, quality=None, type=None, unit=None):
        """
        Gets the amount of data stored for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param quality:  The quality to filter the stats
        :param type:  The type to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send
        :rtype: int
        """
        return service_stats.data_stored(network=self.code, year=year, month=month, quality=quality, type=type, unit=unit)

    def data_stored_buffer(self, year=None, month=None, quality=None, unit=None):
        """
        Gets the amount of 'buffer' data stored for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param quality:  The quality to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send
        :rtype: int
        """
        return self.data_stored(year=year, month=month, quality=quality, type='buffer', unit=unit)

    def data_stored_validated(self, year=None, month=None, quality=None, unit=None):
        """
        Gets the amount of 'validated' data stored for the current network

        :param year: The year to filter the stats
        :param month:  The month to filter the stats
        :param quality:  The quality to filter the stats
        :param unit: The unit to convert the result into (default: 'Bytes')
        :return: The amount of data send
        :rtype: int
        """
        return self.data_stored(year=year, month=month, quality=quality, type='validated', unit=unit)



# encoding: utf8
import logging
import os
import pathlib
import shutil
from jinja2 import Environment, PackageLoader
from . import ServiceReportBuilderAbstract

logger = logging.getLogger(__name__)


class ServiceReportBuilderHtml(ServiceReportBuilderAbstract):
    """
    HTML report builder
    """

    FORMAT_PNG = 'png'
    FORMAT_HTML = 'html'

    @classmethod
    def build(cls, report, output_format=FORMAT_PNG, unit=None, width=None, height=None, use_local_assets=True, with_plots=True):
        """
        Builds an HTML report
        :param report: The report object
        :param output_format: The plot format (default='png')
        :param unit: The unit in which the amount of data are converted
        :param width: The width of generated images
        :param height: The height of generated images
        :param use_local_assets: Enables the usage of local assets instead of on-line assets
        :param with_plots: Include plots
        :return: The output file path
        :rtype: str
        """
        logger.debug("ServiceReportBuilderHtml.build(%s, %s, %s, %s, %s, %s, %s)" % (report, output_format, unit, width, height, use_local_assets, with_plots))

        jinja_env = Environment(loader=PackageLoader('resif_delivery_stats_plotter'),
                                lstrip_blocks=True)  # , trim_blocks=True)

        # Get output directory path
        output_dirpath = cls.output_dir(report, True)

        # Copy assets if needed
        if use_local_assets:
            cls.copy_assets(output_dirpath, (output_format == cls.FORMAT_HTML))

        # Render the page content and get the output filename
        output_filename = None
        if output_format == cls.FORMAT_PNG:
            output = cls.render_png(report, jinja_env, unit, width, height, use_local_assets, with_plots)
            output_filename = 'report_png.html'
        elif output_format == cls.FORMAT_HTML:
            output = cls.render_html(report, jinja_env, unit, width, height, use_local_assets, with_plots)
            output_filename = 'report.html'
        else:
            raise Exception("Format '%s' is not supported by any renderer" % output_format)

        # Write the output content to output file
        output_filepath = os.path.join(output_dirpath, output_filename)
        with open(output_filepath, 'w') as output_file:
            output_file.write(output)

        logger.info("Report saved as %s" % output_filepath)
        return output_filename

    @staticmethod
    def render_png(report, jinja_env, unit, width, height, use_local_assets, with_plots):
        """
        Render an HTML report using static PNG plots
        :param report: The report object
        :param jinja_env: The Jinja2 environment
        :param unit: The unit in which the amount of data are converted
        :param width: The width of generated images
        :param height: The height of generated images
        :param use_local_assets: Enables the usage of local assets instead of on-line assets
        :param with_plots: Include plots
        :return: The HTML report content
        :rtype: str
        """
        logger.debug("ServiceReportBuilderHtml.render_png(%s, %s)" % (report, jinja_env))

        params = {
            'report': report,
            'unit': unit,
            'use_local_assets': use_local_assets,
            'use_plotly': False,
        }
        if with_plots:
            from resif_delivery_stats_plotter.services.plotters.png import ServicePlotterPNG as service_plotter
            params.update({
                'plot_availability_network': service_plotter.plot_network_availability(report.network, report.year, width, height),
                'plot_data_send_yearly': service_plotter.plot_data_send_yearly(report.network, report.year, width, height, unit=unit),
                'plot_data_send_monthly': service_plotter.plot_data_send_monthly(report.network, report.year, width, height, unit=unit),
                'plot_data_stored_yearly': service_plotter.plot_data_stored_yearly(report.network, report.year, width, height, unit=unit),
                'plot_requests_yearly': service_plotter.plot_requests_yearly(report.network, report.year, width, height),
                'plot_requests_monthly': service_plotter.plot_requests_monthly(report.network, report.year, width, height),
                'map_requests_by_country': service_plotter.plot_map_requests_by_country(report.network, report.year, width, height),
                'map_clients_by_country': service_plotter.plot_map_clients_by_country(report.network, report.year, width, height),
                'map_stations': service_plotter.plot_map_network_stations(report.network, report.year, width, height), #FIXME
            })

        jinja_tpl = jinja_env.get_template('report_html_png.jinja2')
        jinja_out = jinja_tpl.render(**params)
        return jinja_out

    @staticmethod
    def render_html(report, jinja_env, unit, width, height, use_local_assets, with_plots):
        """
        Render an HTML report using dynamic JS plots
        :param report: The report object
        :param jinja_env: The Jinja2 environment
        :param unit: The unit in which the amount of data are converted
        :param width: The width of generated images
        :param height: The height of generated images
        :param use_local_assets: Enables the usage of local assets instead of on-line assets
        :param with_plots: Include plots
        :return: The HTML report content
        :rtype: str
        """
        logger.debug("ServiceReportBuilderHtml.render_html(%s, %s)" % (report, jinja_env))

        params = {
            'report': report,
            'unit': unit,
            'use_local_assets': use_local_assets,
            'use_plotly': True,
        }
        if with_plots:
            from resif_delivery_stats_plotter.services.plotters.html import ServicePlotterHTML as service_plotter
            params.update({
                'plot_availability_network': service_plotter.plot_network_availability(report.network, report.year, width, height, output_format=service_plotter.FORMAT_DIV),
                'plot_data_send_yearly': service_plotter.plot_data_send_yearly(report.network, report.year, width, height, output_format=service_plotter.FORMAT_DIV, unit=unit),
                'plot_data_send_monthly': service_plotter.plot_data_send_monthly(report.network, report.year, width, height, output_format=service_plotter.FORMAT_DIV, unit=unit),
                'map_requests_by_country': service_plotter.plot_map_requests_by_country(report.network, report.year, width, height, output_format=service_plotter.FORMAT_DIV),
                'map_clients_by_country': service_plotter.plot_map_clients_by_country(report.network, report.year, width, height, output_format=service_plotter.FORMAT_DIV),
                'map_stations': service_plotter.plot_map_network_stations(report.network, report.year, width, height, output_format=service_plotter.FORMAT_DIV),
            })

        jinja_tpl = jinja_env.get_template('report_html.jinja2')
        jinja_out = jinja_tpl.render(**params)
        return jinja_out

    @staticmethod
    def copy_assets(output_dir, use_plotly=False):
        """
        Copy assets to output directory

        :param output_dir: The output directory path
        """
        logger.debug("ServiceReportBuilderHtml.copy_assets(%s, %s)" % (output_dir, use_plotly))

        # List the assets to copy
        assets = [
            'resif_logo.png',
            'bootstrap.min.css',
            'bootstrap.bundle.min.js',
            'jquery-3.5.1.slim.min.js',
        ]
        if use_plotly:
            assets.append('plotly-latest.min.js')

        # Get the assets source directory path
        root_dir = pathlib.Path(__file__).resolve().parent.parent.parent

        # Copy each asset to output directory
        for asset in assets:
            shutil.copyfile(os.path.join(root_dir, 'assets', asset), os.path.join(output_dir, asset))



# encoding: utf8
import logging
import requests
import arrow
from datetime import date
from resif_delivery_stats_plotter.errors import ApiError, NoDataError
from . import ServiceClientAbstract

logger = logging.getLogger(__name__)


class ServiceClientStatistics(ServiceClientAbstract):
    """
    Client of webservice 'statistics'
    """

    @classmethod
    def _url(cls, query_type, year=None, network=None, station=None, location=None, channel=None, media=None, type=None, country=None, month=None, sum=None, quality=None):
        """
        Builds an URL from the parameters

        :param query_type: The type of query
        :param year: The year to filter
        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param media: The media to filter
        :param type: The type to filter
        :param country: The country code to filter
        :param month: The month to filter
        :param sum: Sum the results
        :param quality: The quality code to filter
        :return: An URL ready to query
        :rtype: str
        """
        logger.debug('ServiceClientStatistics._url(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)' % (query_type, year, network, station, location, channel, media, type, country, month, sum, quality))

        # Parameters
        params = [
            'request=%s' % query_type,
            'format=json'
        ]
        if network:
            params.append('net=%s' % network)
        if station:
            params.append('sta=%s' % station)
        if location:
            params.append('loc=%s' % location)
        if channel:
            params.append('cha=%s' % channel)
        if media:
            params.append('media=%s' % media)
        if type:
            params.append('type=%s' % type)
        if country:
            params.append('country=%s' % country)
        if quality:
            params.append('quality=%s' % quality)
        if sum:
            params.append('sum=true')

        if year and month:
            date_start = date(year, month or 1, 1)
            date_end = arrow.get(date_start).shift(months=1).shift(days=-1).format('YYYY-MM-DD')
            params.append('start=%s' % date_start)
            params.append('end=%s' % date_end)
        elif year:
            params.append('start=%s' % date(year, 1, 1))
            params.append('end=%s' % date(year, 12, 31))

        # Build URL
        return 'http://ws.resif.fr/resifws/statistics/1/query?%s' % '&'.join(params)

    @classmethod
    def _query(cls, query_type, year=None, network=None, station=None, location=None, channel=None, media=None, type=None, country=None, month=None, sum=None, quality=None):
        """
        Run a query

        :param query_type: The type of query
        :param year: The year to filter
        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param media: The media to filter
        :param type: The type to filter
        :param country: The country code to filter
        :param month: The month to filter
        :param sum: Sum the results
        :param quality: The quality code to filter
        :return: The query response
        :rtype: requests.Response
        """
        logger.debug('ServiceClientStatistics._query(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)' % (query_type, year, network, station, location, channel, media, type, country, month, sum, quality))

        # Build URL
        url = cls._url(query_type, network=network, station=station, location=location, channel=channel, year=year, month=month, country=country, media=media, sum=sum, type=type, quality=quality)
        logger.debug('Querying %s', url)

        # Run query
        response = requests.get(url)
        if response.from_cache:
            logger.debug('> Using response from cached request')
        if response.status_code == 200:
            return response.json().get('datasources', {})
        if response.status_code == 204:
            logger.debug('Response %s', response)
            raise NoDataError('GET %s %i' % (url, response.status_code))
        else:
            logger.debug('Response %s', response)
            raise ApiError('GET %s %i' % (url, response.status_code))

    ####################################################################################################################
    # STANDARD QUERIES
    ##################

    @classmethod
    def send(cls, year, network=None, station=None, location=None, channel=None, media=None, month=None, country=None):
        logger.debug('ServiceClientStatistics.send(%s, %s, %s, %s, %s, %s, %s, %s)' % (year, network, station, location, channel, media, month, country))

        # Build URL
        url = cls._url('send', year, network, station, location, channel, media=media, month=month, country=country)
        logger.debug('Querying %s', url)

        # Run query
        response = requests.get(url)
        if response.from_cache:
            logger.debug('> Using response from cached request')
        if response.status_code == 200:
            if network:
                stations = {
                    'all': 0
                }
                for datasource in response.json().get('datasources'):
                    station_send = int(datasource['bytes'])
                    stations[datasource['station']] = station_send
                    stations['all'] += station_send
                return stations
            else:
                return {'all': int(response.json().get('datasources')[0]['bytes'])}
        else:
            logger.debug('Response %s', response)
            # raise ApiError('GET %s %i' % (url, response.status_code))

    @classmethod
    def timeseries(cls, year, network, station=None, location=None, channel=None, media=None, month=None):
        logger.debug('ServiceClientStatistics.timeseries(%s, %s, %s, %s, %s, %s, %s)' % (year, network, station, location, channel, media, month))
        timeseries = {}

        # Build URL
        url = cls._url('timeseries', year, network, station, location, channel, media=media, month=month)
        logger.debug('Querying %s', url)

        # Run query
        response = requests.get(url)
        if response.from_cache:
            logger.debug('> Using response from cached request')
        if response.status_code == 200:
            for datasource in response.json().get('datasources'):
                if datasource['time'] not in timeseries:
                    timeseries[datasource['time']] = {}

                timeseries[datasource['time']][datasource['country']] = {
                    'clients': datasource['clients'],
                    'bytes': datasource['bytes'],
                }

        else:
            logger.debug('Response %s', response)
            # raise ApiError('GET %s %i' % (url, response.status_code))

        return timeseries

    @classmethod
    def country(cls, year, network, station=None, location=None, channel=None, country=None, includes_all=False, month=None):
        logger.debug('ServiceClientStatistics.country(%s, %s, %s, %s, %s, %s, %s, %s)' % (year, network, station, location, channel, country, includes_all, month))
        countries = {}

        # Build URL
        url = cls._url('country', year, network, station, location, channel, country=country, month=month)
        logger.debug('Querying %s', url)

        # Run query
        response = requests.get(url)
        if response.from_cache:
            logger.debug('> Using response from cached request')
        if response.status_code == 200:
            for datasource in response.json().get('datasources'):
                if includes_all or datasource['country'] != 'all':
                    countries[datasource['country']] = {
                        'clients': datasource['clients'],
                        'requests': datasource['requests'],
                    }

        else:
            logger.debug('Response %s', response)
            # raise ApiError('GET %s %i' % (url, response.status_code))

        return countries

    @classmethod
    def storage(cls, year, network, station=None, location=None, channel=None, type='all', month=None):
        logger.debug('ServiceClientStatistics.storage(%s, %s, %s, %s, %s, %s, %s)' % (year, network, station, location, channel, type, month))

        try:
            response = cls._query('storage', network=network, station=station, location=location, channel=channel, year=year, month=month, type=type)
            # TODO
            pass

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    ####################################################################################################################
    # CUSTOM QUERIES
    ################

    @classmethod
    def requests(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country='all', media=None):
        """
        Gets the requests number

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :return: The number of requests
        :rtype: int
        """
        logger.debug('ServiceClientStatistics.requests(%s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media))

        try:
            datasources = cls._query('country', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media)

            total_requests = 0
            for datasource in datasources:
                datasource_requests = datasource.get('requests')
                if datasource_requests not in ('None', None):
                    total_requests += int(datasource_requests)

            return total_requests

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def requests_by_country(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country=None, media=None):
        """
        Gets the requests number by country

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :return: A dictionary of requests number grouped by country
        :rtype: dict
        """
        logger.debug('ServiceClientStatistics.requests_by_country(%s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media))

        try:
            datasources = cls._query('country', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media)

            countries = {}
            for datasource in datasources:
                country_code = datasource.get('country', None)
                country_requests = datasource.get('requests', None)
                if country_code not in ('None', None, 'all') and country_requests not in ('None', None):
                    countries[country_code] = int(country_requests)

            return countries

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def clients(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country='all', media=None):
        """
        Gets the clients number

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :return: The number of clients
        :rtype: int
        """
        logger.debug('ServiceClientStatistics.clients(%s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media))

        try:
            datasources = cls._query('country', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media)

            total_clients = 0
            for datasource in datasources:
                datasource_clients = datasource.get('clients')
                if datasource_clients not in ('None', None):
                    total_clients += int(datasource_clients)

            return total_clients

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def clients_by_country(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country=None, media=None):
        """
        Gets the clients number by country

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :return: A dictionary of clients number grouped country
        :rtype: dict
        """
        logger.debug('ServiceClientStatistics.clients_by_country(%s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media))

        try:
            datasources = cls._query('country', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media)

            countries = {}
            for datasource in datasources:
                country_code = datasource.get('country')
                country_clients = datasource.get('clients')
                if country_code not in ('None', None, 'all') and country_clients not in ('None', None):
                    countries[country_code] = int(country_clients)

            return countries

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def clients_and_requests_by_country(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country=None, media=None):
        """
        Gets the requests and clients number by country

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :return: A dictionary of requests number et clients cumber grouped by country
        :rtype: dict
        """
        logger.debug('ServiceClientStatistics.clients_and_requests_by_country(%s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media))

        try:
            datasources = cls._query('country', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media)

            countries = {}
            for datasource in datasources:
                country_code = datasource.get('country')
                country_clients = datasource.get('clients')
                country_requests = datasource.get('requests')
                if country_code not in ('None', None, 'all') and (country_clients not in ('None', None) or country_requests not in ('None', None)):
                    countries[country_code] = {}
                    if country_clients not in ('None', None):
                        countries[country_code]['clients'] = int(country_clients)
                    if country_requests not in ('None', None):
                        countries[country_code]['requests'] = int(country_requests)

            return countries

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def data_send(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country='all', media=None, unit=None):
        """
        Gets the amount of data send

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :param unit: The unit in which the result is converted ('Bytes' by default)
        :return: The amount of data send
        :rtype: int
        """
        logger.debug('ServiceClientStatistics.data_send(%s, %s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media, unit))

        try:
            datasources = cls._query('send', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media, sum=True)

            total_bytes = 0
            for datasource in datasources:
                datasource_bytes = datasource.get('bytes')
                if datasource_bytes not in ('None', None):
                    if unit:
                        total_bytes += cls.unit_convert(int(datasource_bytes), 'bytes', unit)
                    else:
                        total_bytes += int(datasource_bytes)

            return total_bytes

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def data_send_by_station(cls, network=None, station=None, location=None, channel=None, year=None, month=None, country='all', media=None, unit=None):
        """
        Gets the amount of data send for each station

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param country: The country code to filter
        :param media: The media to filter
        :param unit: The unit in which the result is converted ('Bytes' by default)
        :return: A dictionary of amount of data send grouped by station code
        :rtype: dict
        """
        logger.debug('ServiceClientStatistics.data_send(%s, %s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, country, media, unit))

        try:
            datasources = cls._query('send', network=network, station=station, location=location, channel=channel, year=year, month=month, media=media)

            stations = {}
            for datasource in datasources:
                station_code = datasource.get('station')
                station_bytes = datasource.get('bytes')
                if station_code not in ('None', None, 'all') and station_bytes not in ('None', None):
                    if unit:
                        stations[station_code] = cls.unit_convert(int(station_bytes), 'bytes', unit)
                    else:
                        stations[station_code] = int(station_bytes)

            return stations

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def data_stored(cls, network=None, station=None, location=None, channel=None, year=None, month=None, quality=None, type=None, unit=None):
        """
        Gets the amount of data stored

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality code to filter
        :param type: The type to filter
        :param unit: The unit in which the result is converted ('Bytes' by default)
        :return: The amount of data stored
        :rtype: int
        """
        logger.debug('ServiceClientStatistics.data_stored(%s, %s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, quality, type, unit))

        try:
            datasources = cls._query('storage', network=network, station=station, location=location, channel=channel, year=year, month=month, quality=quality, type=type)

            total_bytes = 0
            for datasource in datasources:
                datasource_size = datasource.get('size')
                if datasource_size not in ('None', None):
                    if unit:
                        total_bytes += cls.unit_convert(int(datasource_size), 'bytes', unit)
                    else:
                        total_bytes += int(datasource_size)

            return total_bytes

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass

    @classmethod
    def data_stored_by_quality(cls, network=None, station=None, location=None, channel=None, year=None, month=None, quality=None, type=None, unit=None):
        """
        Gets the amount of data stored for each quality

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality code to filter
        :param type: The type to filter
        :param unit: The unit in which the result is converted ('Bytes' by default)
        :return: A dictionary of amount of data stored grouped by quality code
        :rtype: dict
        """
        logger.debug('ServiceClientStatistics.data_stored_by_quality(%s, %s, %s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, quality, type, unit))

        try:
            datasources = cls._query('storage', network=network, station=station, location=location, channel=channel, year=year, month=month, quality=quality, type=type)

            qualities = {}
            for datasource in datasources:
                datasource_quality = datasource.get('quality')
                datasource_size = datasource.get('size')
                if datasource_quality not in ('None', None) and datasource_size not in ('None', None):
                    if unit:
                        qualities[datasource_quality] += cls.unit_convert(int(datasource_size), 'bytes', unit)
                    else:
                        qualities[datasource_quality] += int(datasource_size)

            return qualities

        except NoDataError:
            # TODO
            pass
        except ApiError:
            # TODO
            pass



# encoding: utf8
from resif_delivery_stats_plotter.services.report_builders.html import ServiceReportBuilderHtml
from resif_delivery_stats_plotter.errors import NoDataError
from resif_delivery_stats_plotter.factories.network import NetworkFactory


class Report:
    """
    Defines a report
    """

    FORMAT_HTML = 'html'
    FORMAT_PNG = 'png'

    def __init__(self, year=None, network=None, stations=[]):
        self.year = year
        self.network = network
        self._stations = stations

    @property
    def stations(self):
        """
        Gets the list of stations of the current network

        :return: A collection of stations
        :rtype: list
        """
        if self._stations:
            return self._stations
        elif self.network:
            return self.network.stations

    @stations.setter
    def stations(self, value):
        """
        Sets the list of stations

        :param value: The list of station to set
        """
        self._stations = value

    def build(self, output_format=FORMAT_PNG, unit=None, width=None, height=None, with_plots=True):
        """
        Builds the report

        :param output_format: The output format ('png'|'html')
        :param unit: The unit to use for data plots
        :param width: The width of generated images
        :param height: The height of generated images
        :param with_plots: Include plots
        :return: The output filename of the report
        :rtype: str
        """
        if output_format in (self.FORMAT_HTML, self.FORMAT_PNG):
            try:
                return ServiceReportBuilderHtml.build(self, output_format=output_format, unit=unit, width=width, height=height, with_plots=with_plots)
            except NoDataError:
                # TODO
                raise
        else:
            raise Exception("Format '%s' is not supported by any builder" % output_format)

    @property
    def networks(self):
        """
        Gets the list of available networks

        :return: The  list of available networks
        :rtype: list
        """
        return NetworkFactory.list(year=self.year)



# encoding: utf8
from io import open

from setuptools import find_packages, setup

with open('resif_delivery_stats_plotter/__init__.py', 'r') as f:
    for line in f:
        if line.startswith('__version__'):
            version = line.strip().split('=')[1].strip(' \'"')
            break
    else:
        version = '0.1.1'

with open('README.rst', 'r', encoding='utf-8') as f:
    readme = f.read()

REQUIRES = [
    'click',
    'click-log',
    'obspy',
    'requests-cache',
    'ordered_enum',
    'arrow',
    'DateTimeRange',
    'jinja2',
    'pint',
    'matplotlib',
    'pandas',
    'iso3166',
    'staticmap',
    'geopandas',
    'descartes',
    #'folium',
    #'plotly',
    #'kaleido',
]

description = '''resif-delivery-stats-plotter is a command line tool to plot statistics about requests and metadata/data deliveries
by the datacenter about a seismic network handled by Résif. This tool is mostly designed to help the seismic networks' PI
to build their annual activity reports. See http://seismology.resif.fr/'''

kwargs = {
    'name': 'resif-delivery-stats-plotter',
    'version': version,
    'description': 'Résif Delivery stats plotter',
    'long_description': readme,
    'author': 'Philippe Bollard',
    'author_email': 'philippe.bollard@univ-grenoble-alpes.fr',
    'maintainer': 'Philippe Bollard',
    'maintainer_email': 'philippe.bollard@univ-grenoble-alpes.fr',
    'url': 'https://gitlab.com/resif/resif-delivery-stats-plotter',
    'license': 'GPLv3',
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
    ],
    'install_requires': REQUIRES,
    'tests_require': ['coverage', 'pytest'],
    'packages': find_packages(exclude=('tests', 'tests.*')),
    'include_package_data': True,
    'entry_points': '''
        [console_scripts]
        resif-delivery-stats-plotter=resif_delivery_stats_plotter.cli:cli
    ''',

}

setup(**kwargs)


# encoding: utf8
import logging
import arrow
from datetime import date
from obspy.clients.fdsn import Client
from obspy.clients.fdsn.header import FDSNNoDataException, FDSNException
from .station import StationFactory
from .enums import EnumObspyLevel
from ..errors import NoDataError, ApiError
from ..models.network import Network

logger = logging.getLogger(__name__)


class NetworkFactory:
    """
    Factory to builds Network objects from an ObsPy instance
    """

    @classmethod
    def from_obspy(cls, obspy_network, level, channel_year=None):
        """
        Create a Network object from an ObsPy instance

        :param obspy_network: The ObsPy instance
        :param level: The ObsPy query level
        :param channel_year: The year to filter opened channels
        :return: A Network initialized object
        :rtype: Network
        """
        logger.debug('NetworkFactory.from_obspy(%s, %s, %s)' % (obspy_network.code, level, channel_year))
        network = Network(
            code=obspy_network.code,
            start_date=obspy_network.start_date,
            end_date=obspy_network.end_date,
        )

        if level in (EnumObspyLevel.station, EnumObspyLevel.channel):
            for obspy_station in obspy_network.stations:
                network.add_station(StationFactory.from_obspy(obspy_station, level, channel_year, obspy_network.code))

        return network

    @classmethod
    def factory(cls, code_network, level=EnumObspyLevel.station, channel=None, channel_year=None): #FIXME: '?HZ,?NZ'
        """
        Gets a Network object from its code

        :param code_network: The code of the network to get
        :param level: The ObsPy query level (default='station')
        :param channel: The channel code to filter
        :param channel_year: The year to filter opened channels
        :return: A Network initialized object
        :rtype: Network
        """
        logger.debug('NetworkFactory.factory(%s, %s, %s, %s)' % (code_network, level, channel, channel_year))
        try:
            params = {
                'network': code_network,
                'level': level,
                'channel': channel,
            }

            inventory = Client('RESIF').get_stations(**params)
            return cls.from_obspy(inventory[0], level, channel_year)
        except FDSNNoDataException:
            logger.warning("Network %s not found" % code_network)
            raise NoDataError()
        except FDSNException as e:
            raise ApiError(e)

    @classmethod
    def list(cls, level=EnumObspyLevel.network, channel=None, year=None, month=None):
        """
        List the available networks from ObsPy

        :param level: The ObsPy query level (default='network')
        :param channel: The channel code to filter
        :param year: The year to filter opened networks
        :param month: The month to filter opened networks
        :return: The list the available networks from ObsPy
        :rtype: list
        """
        logger.debug('NetworkFactory.list(%s, %s, %s, %s)' % (level, channel, year, month))
        try:
            params = {
                'level': level,
                'channel': channel,
            }

            if year and month:
                date_start = date(year, month or 1, 1)
                date_end = arrow.get(date_start).shift(months=1).shift(days=-1).format('YYYY-MM-DD')
                params['starttime'] = date_start
                params['endtime'] = date_end
            elif year:
                params['starttime'] = date(year, 1, 1)
                params['endtime'] = date(year, 12, 31)

            networks = []
            for item in Client('RESIF').get_stations(**params):
                networks.append(cls.from_obspy(item, level))
            return networks

        except FDSNNoDataException:
            logger.warning("No network found")
            raise NoDataError()
        except FDSNException as e:
            raise ApiError(e)



# encoding: utf8
import logging
import requests
import arrow
from datetime import date
from resif_delivery_stats_plotter.errors import ApiError
from . import ServiceClientAbstract

logger = logging.getLogger(__name__)


class ServiceClientAvailability(ServiceClientAbstract):
    """
    Client of webservice 'availability'
    """

    QUALITY_D_INDETERMINATE = 'D'
    QUALITY_R_RAW = 'R'
    QUALITY_Q_CONTROLLED = 'Q'
    QUALITY_M_MODIFIED = 'M'

    @classmethod
    def _url(cls, network=None, station=None, location=None, channel=None, year=None, month=None, quality=None):
        """
        Builds an URL from the parameters

        :param network: The network code to filter
        :param station: The station code to filter
        :param location: The location code to filter
        :param channel: The channel code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality code to filter
        :return: An URL ready to query
        :rtype: str
        """
        logger.debug('ServiceClientAvailability._url(%s, %s, %s, %s, %s, %s, %s)' % (network, station, location, channel, year, month, quality))

        # Parameters
        params = [
            'format=json'
        ]
        if network:
            params.append('net=%s' % network)
        if station:
            params.append('sta=%s' % station)
        if location:
            params.append('loc=%s' % location)
        if channel:
            params.append('cha=%s' % channel)
        if quality:
            params.append('quality=%s' % channel)

        if year and month:
            date_start = date(year, month or 1, 1)
            date_end = arrow.get(date_start).shift(months=1).shift(days=-1).format('YYYY-MM-DD')
            params.append('start=%s' % date_start)
            params.append('end=%s' % date_end)
        elif year:
            params.append('start=%s' % date(year, 1, 1))
            params.append('end=%s' % date(year, 12, 31))

        # Build URL
        return 'http://ws.resif.fr/fdsnws/availability/1/query?%s' % '&'.join(params)

    @classmethod
    def channel(cls, channel, location, station, network, year, month=None, quality=None):
        """
        Gets the availability of a channel

        :param channel: The channel code to filter
        :param location: The location code to filter
        :param station: The station code to filter
        :param network: The network code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality to filter
        :return: A collection of availability periods grouped by quality
        :rtype: dict
        """
        logger.debug('ServiceClientAvailability.channel(%s, %s, %s, %s, %s, %s, %s)' % (channel, location, station, network, year, month, quality))

        # Build URL
        url = cls._url(network, station, location, channel, year, month=month, quality=quality)
        logger.debug('Querying %s', url)

        # Run query
        response = requests.get(url)
        if response.from_cache:
            logger.debug('> Using response from cached request')
        if response.status_code == 200:
            channel_qualities = {}
            item = response.json()
            for datasource in item.get('datasources'):
                if datasource.get('quality') not in channel_qualities:
                    channel_qualities[datasource.get('quality')] = []

                for timespan in datasource.get('timespans'):
                    channel_qualities[datasource.get('quality')].append(timespan)
                    #print("Availability: from %s to %s" % (timespan[0], timespan[1]))

            return channel_qualities

        else:
            logger.debug('Response %s', response)
            # raise ApiError('GET %s %i' % (url, response.status_code))

    @classmethod
    def check_channel(cls, channel, location, station, network, year, month=None, quality=None):
        """
        Checks the availability of a channel

        :param channel: The channel code to filter
        :param location: The location code to filter
        :param station: The station code to filter
        :param network: The network code to filter
        :param year: The year to filter
        :param month: The month to filter
        :param quality: The quality to filter
        :return: Boolean flag telling if 'availability' stats are available
        :rtype: bool
        """
        logger.debug('ServiceClientAvailability.check_channel(%s, %s, %s, %s, %s, %s, %s)' % (channel, location, station, network, year, month, quality))

        # Build URL
        url = cls._url(network, station, location, channel, year, month=month, quality=quality)
        logger.debug('Querying %s', url)

        # Run query
        response = requests.get(url)
        if response.from_cache:
            logger.debug('> Using response from cached request')

        return response.status_code == 200



# encoding: utf8
import logging
from .network import NetworkFactory
from ..models.report import Report

logger = logging.getLogger(__name__)


class ReportFactory:
    """
    Factory to builds Report objects
    """

    @classmethod
    def factory(cls, year, code_network):
        """
        Created a Report object

        :param year: The year targeted by the report
        :param code_network: The code of the network targeted by the report
        :return: A Report initialized object
        :rtype: Report
        """
        logger.debug('ReportFactory.factory(%s, %s)' % (year, code_network))
        return Report(year=year, network=NetworkFactory.factory(code_network, channel_year=year))
